"""
Usage helper
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""
import sys
import hub.helpers.constants as cte
from typing import Dict


class UsageHelper:
  """
  Usage helper class
  """
  _nrcan_schedule_type_to_hub_schedule_type = {
    'Lighting': cte.LIGHTING,
    'Occupancy': cte.OCCUPANCY,
    'Equipment': cte.APPLIANCES,
    'Thermostat Setpoint Cooling': cte.COOLING_SET_POINT,  # Compose 'Thermostat Setpoint' + 'Cooling'
    'Thermostat Setpoint Heating': cte.HEATING_SET_POINT,  # Compose 'Thermostat Setpoint' + 'Heating'
    'Fan': cte.HVAC_AVAILABILITY
  }
  _nrcan_data_type_to_hub_data_type = {
    'FRACTION': cte.FRACTION,
    'ON_OFF': cte.ON_OFF,
    'TEMPERATURE': cte.ANY_NUMBER
  }

  _nrcan_time_to_hub_time = {
    'Hourly': cte.HOUR,
    'Constant': cte.CONSTANT
  }

  _nrcan_day_type_to_hub_days = {
    'Default|Wkdy': [cte.MONDAY, cte.TUESDAY, cte.WEDNESDAY, cte.THURSDAY, cte.FRIDAY],
    'Sun|Hol': [cte.SUNDAY, cte.HOLIDAY],
    'Sat': [cte.SATURDAY],
    'Default|WntrDsn|SmrDsn': [cte.MONDAY,
                               cte.TUESDAY,
                               cte.WEDNESDAY,
                               cte.THURSDAY,
                               cte.FRIDAY,
                               cte.SATURDAY,
                               cte.SUNDAY,
                               cte.HOLIDAY,
                               cte.WINTER_DESIGN_DAY,
                               cte.SUMMER_DESIGN_DAY],
    'Default': [cte.MONDAY,
                cte.TUESDAY,
                cte.WEDNESDAY,
                cte.THURSDAY,
                cte.FRIDAY,
                cte.SATURDAY,
                cte.SUNDAY,
                cte.HOLIDAY,
                cte.WINTER_DESIGN_DAY,
                cte.SUMMER_DESIGN_DAY]

  }

  _comnet_days = [cte.MONDAY,
                  cte.TUESDAY,
                  cte.WEDNESDAY,
                  cte.THURSDAY,
                  cte.FRIDAY,
                  cte.SATURDAY,
                  cte.SUNDAY,
                  cte.HOLIDAY]

  _comnet_data_type_to_hub_data_type = {
    'Fraction': cte.FRACTION,
    'OnOff': cte.ON_OFF,
    'Temperature': cte.ANY_NUMBER
  }

  _comnet_schedules_key_to_comnet_schedules = {
    'C-1 Assembly': 'C-1 Assembly',
    'C-2 Public': 'C-2 Health',
    'C-3 Hotel Motel': 'C-3 Hotel',
    'C-4 Manufacturing': 'C-4 Manufacturing',
    'C-5 Office': 'C-5 Office',
    'C-7 Restaurant': 'C-7 Restaurant',
    'C-8 Retail': 'C-8 Retail',
    'C-9 Schools': 'C-9 School',
    'C-10 Warehouse': 'C-10 Warehouse',
    'C-11 Laboratory': 'C-11 Lab',
    'C-12 Residential': 'C-12 Residential',
    'C-14 Gymnasium': 'C-14 Gymnasium'
  }

  @property
  def nrcan_day_type_to_hub_days(self):
    return self._nrcan_day_type_to_hub_days

  @property
  def nrcan_schedule_type_to_hub_schedule_type(self):
    return self._nrcan_schedule_type_to_hub_schedule_type

  @property
  def nrcan_data_type_to_hub_data_type(self):
    return self._nrcan_data_type_to_hub_data_type

  @property
  def nrcan_time_to_hub_time(self):
    return self._nrcan_time_to_hub_time

  @property
  def comnet_data_type_to_hub_data_type(self):
    return self._comnet_data_type_to_hub_data_type

  @property
  def comnet_schedules_key_to_comnet_schedules(self) -> Dict:
    return self._comnet_schedules_key_to_comnet_schedules

  @property
  def comnet_days(self):
    return self._comnet_days

  @staticmethod
  def schedules_key(usage):
    """
    Get Comnet schedules key from the list found in the Comnet usage file
    :param usage: str
    :return: str
    """
    try:
      return UsageHelper._comnet_schedules_key_to_comnet_schedules[usage]
    except KeyError:
      sys.stderr.write('Error: Comnet keyword not found. An update of the Comnet files might have been '
                       'done changing the keywords.\n')



"""
Greenery catalog data model Vegetation class
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez guillermo.gutierrezmorote@concordia.ca
"""

from hub.catalog_factories.data_models.greenery.plant_percentage import PlantPercentage


class Vegetation:
  def __init__(self, category, vegetation, plant_percentages):
    self._name = vegetation.name
    self._category = category
    self._soil_thickness = vegetation.thicknessOfSoil
    self._management = vegetation.management
    self._air_gap = vegetation.airGap
    self._soil_name = vegetation.soil.name
    self._soil_roughness = vegetation.soil.roughness
    self._dry_soil_conductivity = vegetation.soil.conductivityOfDrySoil
    self._dry_soil_density = vegetation.soil.densityOfDrySoil
    self._dry_soil_specific_heat = vegetation.soil.specificHeatOfDrySoil
    self._soil_thermal_absorptance = vegetation.soil.thermalAbsorptance
    self._soil_solar_absorptance = vegetation.soil.solarAbsorptance
    self._soil_visible_absorptance = vegetation.soil.visibleAbsorptance
    self._soil_saturation_volumetric_moisture_content = vegetation.soil.saturationVolumetricMoistureContent
    self._soil_residual_volumetric_moisture_content = vegetation.soil.residualVolumetricMoistureContent
    self._soil_initial_volumetric_moisture_content = vegetation.soil.initialVolumetricMoistureContent
    self._plant_percentages = plant_percentages

  @property
  def name(self):
    """
    Get vegetation name
    :return: string
    """
    return self._name

  @property
  def category(self):
    """
    Get vegetation category
    :return: string
    """
    return self._category

  @property
  def soil_thickness(self):
    """
    Get soil thickness in m
    :return: float
    """
    return self._soil_thickness

  @property
  def management(self):
    """
    Get management
    :return: string
    """
    return self._management

  @property
  def air_gap(self):
    """
    Get air gap in m
    :return: float
    """
    return self._air_gap

  @property
  def plant_percentages(self) -> [PlantPercentage]:
    """
    Get plant percentages
    :return: [PlantPercentage]
    """
    percentage = 0.0
    for plant_percentage in self._plant_percentages:
      percentage += float(plant_percentage.percentage)
    if percentage > 100:
      raise ValueError('the plant percentage in this vegetation is over 100%')
    return self._plant_percentages

  @property
  def soil_name(self):
    """
    Get soil name
    :return: string
    """
    return self._soil_name

  @property
  def soil_roughness(self):
    """
    Get soil roughness
    :return: float
    """
    return self._soil_roughness

  @property
  def dry_soil_conductivity(self):
    """
    Get soil dry conductivity in W/mK
    :return: float
    """
    return self._dry_soil_conductivity

  @property
  def dry_soil_density(self):
    """
    Get soil dry density in kg/m3
    :return: float
    """
    return self._dry_soil_density

  @property
  def dry_soil_specific_heat(self):
    """
    Get soil dry specific heat in J/kgK
    :return: float
    """
    return self._dry_soil_specific_heat

  @property
  def soil_thermal_absorptance(self):
    """
    Get soil thermal absortance
    :return: float
    """
    return self._soil_thermal_absorptance

  @property
  def soil_solar_absorptance(self):
    """
    Get soil solar absortance
    :return: float
    """
    return self._soil_solar_absorptance

  @property
  def soil_visible_absorptance(self):
    """
    Get soil visible absortance
    :return: float
    """
    return self._soil_visible_absorptance

  @property
  def soil_saturation_volumetric_moisture_content(self):
    """
    Get soil saturation volumetric moisture content
    :return: float
    """
    return self._soil_saturation_volumetric_moisture_content

  @property
  def soil_residual_volumetric_moisture_content(self):
    """
    Get soil residual volumetric moisture content
    :return: float
    """
    return self._soil_residual_volumetric_moisture_content

  @property
  def soil_initial_volumetric_moisture_content(self):
    """
    Get soil initial volumetric moisture content
    :return: float
    """
    return self._soil_initial_volumetric_moisture_content



"""
Dictionaries module for Montreal function to hub function
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez Guillermo.GutierrezMorote@concordia.ca
"""

import hub.helpers.constants as cte


class MontrealFunctionToHubFunction:

  def __init__(self):
    self._dictionary = {
      "Administration publique municipale et régionale": cte.MEDIUM_OFFICE,
      "Administration publique provinciale": cte.MEDIUM_OFFICE,
      "Agence de voyages ou d'expéditions": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Aiguillage et cour de triage de chemins de fer": cte.WAREHOUSE,
      "Amphithéâtre et auditorium": cte.EVENT_LOCATION,
      "Archives (incluant cinémathèquevidéothèque)": cte.EVENT_LOCATION,
      "Aréna et activités connexes (patinage sur glace)": cte.SPORTS_LOCATION,
      "Association civiquesociale et fraternelle": cte.OFFICE_AND_ADMINISTRATION,
      "Associationunion ou coop d'épargne et de prêt (inclus caisses populaires locales)": cte.OFFICE_AND_ADMINISTRATION,
      "Atelier d'artiste": cte.WAREHOUSE,
      "Atelier d'artiste ou d'artisan": cte.WAREHOUSE,
      "Atelier d'usinage": cte.WAREHOUSE,
      "Atelier de mécanicien-dentiste": cte.WAREHOUSE,
      "Auberge ou gîte touristique (Hôtel à caractère familiald'au plus 3 étages en hauteur de bâtiment)": cte.SMALL_HOTEL,
      "Autoroute": cte.WAREHOUSE,
      "Autres activités agricoles": cte.INDUSTRY,
      "Autres activités culturelles": cte.EVENT_LOCATION,
      "Autres activités d'hébergement": cte.MULTI_FAMILY_HOUSE,
      "Autres activités d'impression commerciale": cte.WAREHOUSE,
      "Autres activités de la restauration": cte.WAREHOUSE,
      "Autres activités de récupération et de triage": cte.WAREHOUSE,
      "Autres activités de vente au détail (inclus les kiosques d'autres choses que vêtements et accessoires de vêtements)": cte.STAND_ALONE_RETAIL,
      "Autres activités de vente au détail de produits de l'alimentation": cte.STAND_ALONE_RETAIL,
      "Autres activités de vente au détail de vêtements comme les accessoires": cte.STAND_ALONE_RETAIL,
      "Autres activités de vente au détail reliées aux automobilesaux embarcationsaux avions et à leurs accessoires": cte.STAND_ALONE_RETAIL,
      "Autres activités de vente en gros": cte.WAREHOUSE,
      "Autres activités minières et extraction de carrières de minerais non métalliques (sauf le pétrole)": cte.INDUSTRY,
      "Autres activités nautiques": cte.WAREHOUSE,
      "Autres activités religieuses": cte.OFFICE_AND_ADMINISTRATION,
      "Autres activités reliées au transport de matériaux par camion": cte.WAREHOUSE,
      "Autres activités reliées au transport par autobus": cte.WAREHOUSE,
      "Autres activités reliées au transport par chemin de fer": cte.WAREHOUSE,
      "Autres activités sportives (inclus centres de tir à l'arc)": cte.SPORTS_LOCATION,
      "Autres aménagements d'assemblées publiques": cte.OFFICE_AND_ADMINISTRATION,
      "Autres aménagements publics pour différentes activités": cte.OFFICE_AND_ADMINISTRATION,
      "Autres aéroports": cte.WAREHOUSE,
      "Autres bases et réserves militaires": cte.WAREHOUSE,
      "Autres centres de recherche": cte.SECONDARY_SCHOOL,
      "Autres centres de services sociaux ou bureaux de travailleurs sociaux": cte.OFFICE_AND_ADMINISTRATION,
      "Autres centres et réseaux de télévision et de radiodiffusion (système combiné)": cte.MEDIUM_OFFICE,
      "Autres entreposages": cte.WAREHOUSE,
      "Autres espaces de plancher inoccupé": cte.WAREHOUSE,
      "Autres espaces de terrain et étendues d'eau inexploités": cte.WAREHOUSE,
      "Autres expositions d'objets culturels": cte.EVENT_LOCATION,
      "Autres immeubles résidentiels": cte.MID_RISE_APARTMENT,
      "Autres industries d'appareils d'éclairage": cte.INDUSTRY,
      "Autres industries de boissons": cte.INDUSTRY,
      "Autres industries de la fabrication d'éléments de charpentes métalliques": cte.INDUSTRY,
      "Autres industries de la fonte et de l'affinage de métaux non-ferreux": cte.INDUSTRY,
      "Autres industries de la machinerie industrielle et de l'équipement industriel": cte.INDUSTRY,
      "Autres industries de pièces et d'accessoires pour véhicules automobiles": cte.INDUSTRY,
      "Autres industries de produits alimentaires": cte.INDUSTRY,
      "Autres industries de produits alimentaires à base de fruits et de légumes": cte.INDUSTRY,
      "Autres industries de produits chimiques": cte.INDUSTRY,
      "Autres industries de produits du pétrole et du charbon": cte.INDUSTRY,
      "Autres industries de produits en béton": cte.INDUSTRY,
      "Autres industries de produits en caoutchouc": cte.INDUSTRY,
      "Autres industries de produits en fil métallique": cte.INDUSTRY,
      "Autres industries de produits en plastique": cte.INDUSTRY,
      "Autres industries de produits manufacturés": cte.INDUSTRY,
      "Autres industries de produits métalliques d'ornement et d'architecture": cte.INDUSTRY,
      "Autres industries de produits métalliques divers": cte.INDUSTRY,
      "Autres industries de produits textiles": cte.INDUSTRY,
      "Autres industries de produits électriques.": cte.INDUSTRY,
      "Autres industries de vêtements coupés cousus pour femmes et filles": cte.INDUSTRY,
      "Autres industries du bois": cte.INDUSTRY,
      "Autres industries du laminagedu moulage et de l'extrusion de métaux non-ferreux": cte.INDUSTRY,
      "Autres industries du matériel de transport": cte.INDUSTRY,
      "Autres industries du matériel scientifique et professionnel": cte.INDUSTRY,
      "Autres industries du matériel électrique d'usage industriel": cte.INDUSTRY,
      "Autres industries du matériel électronique et de communication": cte.INDUSTRY,
      "Autres industries du meuble de bureau": cte.INDUSTRY,
      "Autres industries du meuble et d'articles d'ameublement": cte.INDUSTRY,
      "Autres industries du meuble résidentiel.": cte.INDUSTRY,
      "Autres industries du papier": cte.INDUSTRY,
      "Autres industries sidérurgiques": cte.INDUSTRY,
      "Autres infrastructures de transport maritime": cte.INDUSTRY,
      "Autres installations inhérentes aux ordures": cte.WAREHOUSE,
      "Autres installations pour les sports": cte.SPORTS_LOCATION,
      "Autres institutions de formation spécialisée (inclus écoles de langues de coutured'arts martiaux de combats et autres)": cte.SECONDARY_SCHOOL,
      "Autres lieux d'assemblée pour les loisirs": cte.OFFICE_AND_ADMINISTRATION,
      "Autres locaux de groupes": cte.OFFICE_AND_ADMINISTRATION,
      "Autres maisons d'institutions religieuses": cte.OFFICE_AND_ADMINISTRATION,
      "Autres maisons et locaux fraternels": cte.OFFICE_AND_ADMINISTRATION,
      "Autres maisons pour personnes retraitées": cte.OFFICE_AND_ADMINISTRATION,
      "Autres parcs": cte.WAREHOUSE,
      "Autres routes et voies publiques": cte.WAREHOUSE,
      "Autres résidences d'étudiants": cte.DORMITORY,
      "Autres résidences provisoires": cte.DORMITORY,
      "Autres services connexes aux valeurs mobilières et aux marchandises": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Autres services d'affaires": cte.MEDIUM_OFFICE,
      "Autres services d'aqueduc et d'irrigation": cte.WAREHOUSE,
      "Autres services de construction de bâtiments": cte.WAREHOUSE,
      "Autres services de génie civil (entrepreneur général)": cte.WAREHOUSE,
      "Autres services de l'automobile": cte.WAREHOUSE,
      "Autres services de location (sauf entreposage)": cte.WAREHOUSE,
      "Autres services de nettoyage": cte.WAREHOUSE,
      "Autres services de réparation et d'entretien d'articles personnels et ménagers": cte.STAND_ALONE_RETAIL,
      "Autres services de soins thérapeutiques": cte.OUT_PATIENT_HEALTH_CARE,
      "Autres services de travaux de construction spécialisés": cte.WAREHOUSE,
      "Autres services de travaux de finition de bâtiment (entrepreneur spécialisé)": cte.WAREHOUSE,
      "Autres services de télécommunications": cte.STAND_ALONE_RETAIL,
      "Autres services divers": cte.WAREHOUSE,
      "Autres services du pétrole": cte.WAREHOUSE,
      "Autres services gouvernementaux": cte.OFFICE_AND_ADMINISTRATION,
      "Autres services immobiliersfinanciers et d'assurance": cte.OFFICE_AND_ADMINISTRATION,
      "Autres services médicaux et de santé": cte.OUT_PATIENT_HEALTH_CARE,
      "Autres services personnels": cte.OUT_PATIENT_HEALTH_CARE,
      "Autres services pour animaux domestiques": cte.OUT_PATIENT_HEALTH_CARE,
      "Autres services pour le transport": cte.WAREHOUSE,
      "Autres services pour les bâtiments": cte.WAREHOUSE,
      "Autres services professionnels": cte.OFFICE_AND_ADMINISTRATION,
      "Autres services publics (infrastructure)": cte.WAREHOUSE,
      "Autres services reliés à la foresterie": cte.WAREHOUSE,
      "Autres terrains de jeux et pistes athlétiques": cte.SPORTS_LOCATION,
      "Autres transports par avion (infrastructure)": cte.WAREHOUSE,
      "Autres transports par véhicule automobile": cte.WAREHOUSE,
      "Autres transportscommunications et services publics (infrastructure)": cte.GREEN_HOUSE,
      "Autres types de production végétale": cte.GREEN_HOUSE,
      "Autres ventes au détail de marchandises en général": cte.STAND_ALONE_RETAIL,
      "Autres établissements avec service complet ou restreint": cte.STAND_ALONE_RETAIL,
      "Autres établissements de débits de boissons alcoolisées": cte.STAND_ALONE_RETAIL,
      "Aéroport et aérodrome": cte.EVENT_LOCATION,
      "Bar à crème glacée": cte.QUICK_SERVICE_RESTAURANT,
      "Bar à spectacles": cte.FULL_SERVICE_RESTAURANT,
      "Bibliothèque": cte.OFFICE_AND_ADMINISTRATION,
      "Bureau de poste": cte.OFFICE_AND_ADMINISTRATION,
      "Bâtiment incendié et inutilisable": cte.NON_HEATED,
      "C.E.G.E.P. (collège d'enseignement général et professionnel)": cte.SECONDARY_SCHOOL,
      "Centre commercial de quartier (15 à 44 magasins)": cte.STRIP_MALL,
      "Centre commercial de voisinage (14 magasins et moins)": cte.STAND_ALONE_RETAIL,
      "Centre commercial local (45 à 99 magasins)": cte.STRIP_MALL,
      "Centre commercial régional (100 à 199 magasins)": cte.STRIP_MALL,
      "Centre commercial super régional (200 magasins et plus)": cte.STRIP_MALL,
      "Centre communautaire ou de quartier (inclus Centre diocésain)": cte.OFFICE_AND_ADMINISTRATION,
      "Centre d'accueil ou établissement curatif (inclus centre de réadaptation pour handicapés physiques et mentaux)": cte.OUT_PATIENT_HEALTH_CARE,
      "Centre d'appels téléphoniques": cte.MEDIUM_OFFICE,
      "Centre d'entraide et de ressources communautaires (inclus ressources d'hébergement de meubles et d'alimentation)": cte.OUT_PATIENT_HEALTH_CARE,
      "Centre d'entreposage de produits pétroliers (pétrole brutgaz pétrole liquéfiémazout domestique et autres produits raffinés)": cte.WAREHOUSE,
      "Centre d'entreposage du gaz (avant distrib.aux consommateurs)": cte.WAREHOUSE,
      "Centre de distribution ou d'expédition de marchandises diverses": cte.WAREHOUSE,
      "Centre de recherche d'activités émergentes (inclus technologies langagières et la photonique)": cte.SECONDARY_SCHOOL,
      "Centre de santé (inclus saunas spas et bains thérapeutiques ou turcs)": cte.OUT_PATIENT_HEALTH_CARE,
      "Centre de services sociaux (C.S.S. et C.R.S.S.S.)": cte.OUT_PATIENT_HEALTH_CARE,
      "Centre de transfert ou d'entreposage de déchets dangereux": cte.WAREHOUSE,
      "Centre de tri postal": cte.WAREHOUSE,
      "Centre de vérification technique d'automobiles et d'estimation": cte.WAREHOUSE,
      "Centre local de services communautaires (C.L.S.C.)": cte.OFFICE_AND_ADMINISTRATION,
      "Centre militaire de transport et d'entreposage": cte.WAREHOUSE,
      "Centre récréatif en général (activités récréatives diversifiées pour tous groupes d'âge)": cte.EVENT_LOCATION,
      "Centre sportif multidisciplinaire (couvert).": cte.SPORTS_LOCATION,
      "Chalet ou maison de villégiature": cte.SINGLE_FAMILY_HOUSE,
      "Chemin de fer (sauf train touristiqueaiguillage et cour de triage)": cte.WAREHOUSE,
      "Cimetière": cte.WAREHOUSE,
      "Cinéma": cte.EVENT_LOCATION,
      "Clinique médicale (cabinet de médecins généralistes)": cte.OUT_PATIENT_HEALTH_CARE,
      "Commission scolaire": cte.OFFICE_AND_ADMINISTRATION,
      "Conserveriemarinagesaumurage et séchage de fruits et de légumes": cte.WAREHOUSE,
      "Construction d'immeubles pour revente": cte.WAREHOUSE,
      "Couvent": cte.EVENT_LOCATION,
      "Dépanneur (sans vente d'essence)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Entreposage de tout genre": cte.WAREHOUSE,
      "Entreposage du mobilier et d'appareils ménagersincluant les mini-entrepôts": cte.WAREHOUSE,
      "Entreposage en vrac à l'extérieur": cte.WAREHOUSE,
      "Entreposage frigorifique (sauf les armoires frigorifiques)": cte.WAREHOUSE,
      "Entreprise d'excavationde nivellementde défrichage et installation de fosses septiques": cte.WAREHOUSE,
      "Entrepôt pour le transport par camion": cte.WAREHOUSE,
      "Entretien et équipement de chemins de fer": cte.WAREHOUSE,
      "Espace de plancher inoccupé dont l'usage serait commercial autre": cte.NON_HEATED,
      "Espace de plancher inoccupé dont l'usage serait industriel": cte.NON_HEATED,
      "Espace de plancher inoccupé dont l'usage serait pour des fins culturelles": cte.NON_HEATED,
      "Espace de plancher inoccupé dont l'usage serait pour services publics": cte.NON_HEATED,
      "Espace de rangement (condo non résidentiel)": cte.NON_HEATED,
      "Espace de rangement (condo)": cte.NON_HEATED,
      "Espace de terrain non aménagé et non exploité (sauf l'exploitation non commerciale de la forêt)": cte.NON_HEATED,
      "Espace pour le séchage des boues provenant de l'usine d'épuration": cte.WAREHOUSE,
      "Fabrication de crème glacée et de desserts congelés": cte.INDUSTRY,
      "Fondations et organismes de charité": cte.OFFICE_AND_ADMINISTRATION,
      "Galerie d'art": cte.EVENT_LOCATION,
      "Garage d'autobus et équipement d'entretien": cte.WAREHOUSE,
      "Garage de stationnement pour automobiles (infrastructure)": cte.WAREHOUSE,
      "Garage de stationnement pour véhicules lourds (Infrastructure)": cte.WAREHOUSE,
      "Garage et équipement d'entretien pour le transport par camion (incluant garages municipaux)": cte.WAREHOUSE,
      "Gare d'autobus pour passagers": cte.WAREHOUSE,
      "Gare de chemins de fer": cte.WAREHOUSE,
      "Gymnase et formation athlétique": cte.SPORTS_LOCATION,
      "Hangar à avion": cte.WAREHOUSE,
      "Hôtel (incluant les hôtels-motels)": cte.SMALL_HOTEL,
      "Hôtel résidentiel": cte.SMALL_HOTEL,
      "Immeuble commercial": cte.STAND_ALONE_RETAIL,
      "Immeuble non résidentiel en construction": cte.NON_HEATED,
      "Immeuble résidentiel en construction": cte.RESIDENTIAL,
      "Immeuble à bureaux": cte.MEDIUM_OFFICE,
      "Immeuble à temps partagé («time share») Propriété ou copropriété ou groupe d'usufruitier ont chacun droit de jouissancepériodique et successif.": cte.MEDIUM_OFFICE,
      "Incinérateur": cte.INDUSTRY,
      "Industrie d'accessoires vestimentaires et d'autres vêtements": cte.INDUSTRY,
      "Industrie d'alcools destinés à la consommation (distillerie)": cte.INDUSTRY,
      "Industrie d'appareils d'éclairage (sauf ampoules et tubes)": cte.INDUSTRY,
      "Industrie d'appareils orthopédiques et chirurgicaux": cte.INDUSTRY,
      "Industrie d'armoires de placards de cuisine et de coiffeuses de salle de bains en bois": cte.INDUSTRY,
      "Industrie d'articles de maison en textile et d'articles d'hygiène en textile": cte.INDUSTRY,
      "Industrie d'articles de sport et d'athlétisme": cte.INDUSTRY,
      "Industrie d'assaisonnements et de vinaigrettes": cte.INDUSTRY,
      "Industrie d'autres produits de boulangerie et de pâtisseries": cte.INDUSTRY,
      "Industrie d'autres vêtements coupés cousus pour hommes et garçons": cte.INDUSTRY,
      "Industrie d'engrais chimique et d'engrais composé": cte.INDUSTRY,
      "Industrie d'enseignes au néon (excluant les enseignes en bois) éclairage interne": cte.INDUSTRY,
      "Industrie d'équipements de télécommunication": cte.INDUSTRY,
      "Industrie de bas et de chaussettes": cte.INDUSTRY,
      "Industrie de boissons gazeuses": cte.INDUSTRY,
      "Industrie de boîtes en carton ondulé et en carton compact": cte.INDUSTRY,
      "Industrie de boîtes pliantes et rigides": cte.INDUSTRY,
      "Industrie de carrosseries de véhicules automobiles": cte.INDUSTRY,
      "Industrie de chaudièresd'échangeurs de chaleur et de plaques métalliques": cte.INDUSTRY,
      "Industrie de contenants en plastique (sauf en mousse)": cte.INDUSTRY,
      "Industrie de contreplaqués en bois": cte.INDUSTRY,
      "Industrie de fabrication de gaz industriel": cte.INDUSTRY,
      "Industrie de fils et de câbles électriques": cte.INDUSTRY,
      "Industrie de filés et de tissus tissés (coton)": cte.INDUSTRY,
      "Industrie de garnitures et de raccords de plomberie en métal": cte.INDUSTRY,
      "Industrie de jouets et de jeux": cte.INDUSTRY,
      "Industrie de l'abattage et du conditionnement de la viande (sauf la volaille et le petit gibier)": cte.INDUSTRY,
      "Industrie de l'abattage et du conditionnement de la volaille et du petit gibier": cte.INDUSTRY,
      "Industrie de l'impression de formulaires commerciaux": cte.INDUSTRY,
      "Industrie de l'équipement de manutention": cte.INDUSTRY,
      "Industrie de l'étirage de l'extrusion et alliage de l'aluminiumfabriqué à partir d'aluminium acheté": cte.INDUSTRY,
      "Industrie de la bijouterie et de l'orfèvrerie (sauf l'affinage secondaire de métaux précieux)": cte.INDUSTRY,
      "Industrie de la bière": cte.INDUSTRY,
      "Industrie de la chaussure": cte.INDUSTRY,
      "Industrie de la confection à forfait de vêtements pour femmes et filles": cte.INDUSTRY,
      "Industrie de la construction et de la réparation de navires": cte.INDUSTRY,
      "Industrie de la fabrication de supports d'enregistrement de la reproduction du son et des instruments de musique": cte.INDUSTRY,
      "Industrie de la glace": cte.INDUSTRY,
      "Industrie de la machinerie pour la construction et du matériel d'entretien": cte.INDUSTRY,
      "Industrie de la préparation et du conditionnement de poissons et de fruits de mer": cte.INDUSTRY,
      "Industrie de la quincaillerie de base": cte.INDUSTRY,
      "Industrie de la transformation de la viande et de la fonte des graisses animales": cte.INDUSTRY,
      "Industrie de la tôlerie pour ventilation": cte.INDUSTRY,
      "Industrie de lampes électriques (ampoules et tubes)": cte.INDUSTRY,
      "Industrie de moteurs et de pièces de moteurs de véhicules automobiles": cte.INDUSTRY,
      "Industrie de mélange de farine et de pâte": cte.INDUSTRY,
      "Industrie de peinturede teinture et de vernis": cte.INDUSTRY,
      "Industrie de pellicules et de feuilles non renforcées en plastique": cte.INDUSTRY,
      "Industrie de pièces en plastique pour véhicules automobiles": cte.INDUSTRY,
      "Industrie de pièces et de composantes électroniques": cte.INDUSTRY,
      "Industrie de pneus et de chambres à air": cte.INDUSTRY,
      "Industrie de portes et de fenêtres en métal": cte.INDUSTRY,
      "Industrie de portes et fenêtres en plastique": cte.INDUSTRY,
      "Industrie de produits chimiques inorganiques d'usage industriel": cte.INDUSTRY,
      "Industrie de produits d'architecture en plastique": cte.INDUSTRY,
      "Industrie de produits de boulangerie commerciale de produits de boulangerie congelés et de pâtisseries": cte.INDUSTRY,
      "Industrie de produits de toilette": cte.INDUSTRY,
      "Industrie de produits en pierre": cte.INDUSTRY,
      "Industrie de produits en plastique stratifié sous pression ou renforcé": cte.INDUSTRY,
      "Industrie de produits en verre fabriqué à partir de verre acheté": cte.INDUSTRY,
      "Industrie de produits pharmaceutiques et de médicaments": cte.INDUSTRY,
      "Industrie de produits pétrochimiques": cte.INDUSTRY,
      "Industrie de produits pétroliers raffinés (sauf les huiles de graissage et les graisses lubrifiantes)": cte.INDUSTRY,
      "Industrie de pâtes alimentaires sèches": cte.INDUSTRY,
      "Industrie de récipients et de boîtes en métal": cte.INDUSTRY,
      "Industrie de résines synthétiques et de caoutchouc synthétique": cte.INDUSTRY,
      "Industrie de sacs et de poches en matière textile": cte.INDUSTRY,
      "Industrie de sacs et de sachets en plastique": cte.INDUSTRY,
      "Industrie de savons et de détachants pour le nettoyage": cte.INDUSTRY,
      "Industrie de sommiers et de matelas": cte.INDUSTRY,
      "Industrie de soupapes en métal": cte.INDUSTRY,
      "Industrie de tapis carpettes et moquettes": cte.INDUSTRY,
      "Industrie de tous les autres produits divers en bois": cte.INDUSTRY,
      "Industrie de tous les autres produits en papier transformé (sauf pour le bureau)": cte.INDUSTRY,
      "Industrie de ventilateursde soufflantes et de purificateurs d'air industriels et commerciaux": cte.INDUSTRY,
      "Industrie de vêtements de sport pour femmes et filles": cte.INDUSTRY,
      "Industrie de vêtements professionnels coupés cousus": cte.INDUSTRY,
      "Industrie des pièces et accessoires d'aéronefs (incluant avions et hélicoptères)": cte.INDUSTRY,
      "Industrie du béton préparé": cte.INDUSTRY,
      "Industrie du cannabis": cte.INDUSTRY,
      "Industrie du ciment": cte.INDUSTRY,
      "Industrie du clichagede la composition de la reliure et de la lithographie": cte.INDUSTRY,
      "Industrie du fromage": cte.INDUSTRY,
      "Industrie du lait de consommation": cte.INDUSTRY,
      "Industrie du laminagede l'étirage et de l'extrusion du cuivre et de ses alliages": cte.INDUSTRY,
      "Industrie du matériel de chauffage et du matériel de réfrigération commerciale": cte.INDUSTRY,
      "Industrie du matériel de transport": cte.INDUSTRY,
      "Industrie du matériel ferroviaire roulant": cte.INDUSTRY,
      "Industrie du matériel électrique de communication et de protection": cte.INDUSTRY,
      "Industrie du meuble de maison en bois": cte.INDUSTRY,
      "Industrie du meuble et d'articles d'ameublement pour hôtelsrestaurants et institutions": cte.INDUSTRY,
      "Industrie du pain": cte.INDUSTRY,
      "Industrie du revêtement métallique sur commande": cte.INDUSTRY,
      "Industrie du sucre de canne et de betterave à sucre": cte.INDUSTRY,
      "Industrie du thé et du café": cte.INDUSTRY,
      "Industries des appareils d'aéronefs (incluant avions et hélicoptères)": cte.INDUSTRY,
      "Installation d'équipements de réfrigération commerciale": cte.WAREHOUSE,
      "Installation portuaire en général": cte.WAREHOUSE,
      "Jardin botanique": cte.WAREHOUSE,
      "Ligne de l'oléoduc": cte.WAREHOUSE,
      "Local pour les associations fraternelles": cte.EVENT_LOCATION,
      "Logement": cte.RESIDENTIAL,
      "Logement vacant dans un bâtiment comportant plusieurs logements ou autres locaux": cte.RESIDENTIAL,
      "Loisir et autres activités culturelles": cte.HALL,
      "Maison d'agents de courtiers et de services d'administration des biens-fonds": cte.MEDIUM_OFFICE,
      "Maison d'étudiants (collège et université)": cte.DORMITORY,
      "Maison de chambres et pension": cte.DORMITORY,
      "Maison de chambres pour personnes ayant une déficience intellectuelle": cte.DORMITORY,
      "Maison de courtiers et de négociants de marchandises": cte.MEDIUM_OFFICE,
      "Maison de réhabilitation": cte.HOSPITAL,
      "Maison des jeunes": cte.HALL,
      "Maison pour personnes en difficulté (séjours périodes limitées)": cte.HOSPITAL,
      "Maison pour personnes retraitées autonomes": cte.DORMITORY,
      "Maison pour personnes retraitées non autonomes (inclus les CHLSD)": cte.DORMITORY,
      "Marché public": cte.STRIP_MALL,
      "Meunerie et minoterie": cte.MEDIUM_OFFICE,
      "Monastère": cte.DORMITORY,
      "Monument et site historique": cte.EVENT_LOCATION,
      "Motel": cte.SMALL_HOTEL,
      "Musée": cte.EVENT_LOCATION,
      "Organisme international et autres organismes extraterritoriaux": cte.MEDIUM_OFFICE,
      "Parc d'amusement (extérieur)": cte.NON_HEATED,
      "Parc de maisons mobiles (fonds de terre seulement)": cte.NON_HEATED,
      "Parc pour la récréation en général": cte.NON_HEATED,
      "Parc à caractère récréatif et ornemental": cte.NON_HEATED,
      "Passage": cte.NON_HEATED,
      "Piscine extérieure et activités connexes": cte.NON_HEATED,
      "Piscine intérieure et activités connexes": cte.SPORTS_LOCATION,
      "Pose et réparation de parement métalliques et autres (entrepreneur spécialisé)": cte.WAREHOUSE,
      "Poste et bureau de douanes": cte.OFFICE_AND_ADMINISTRATION,
      "Pouponnière ou garderie de nuit": cte.HOSPITAL,
      "Presbytère": cte.OFFICE_AND_ADMINISTRATION,
      "Prison provinciale": cte.DORMITORY,
      "Protection contre l'incendie et activités connexes": cte.WAREHOUSE,
      "Raffinerie de pétrole": cte.INDUSTRY,
      "Restaurant et établissement avec service complet (avec terrasse) - Établissements avec permis alcool inclus pub café et brasserie": cte.FULL_SERVICE_RESTAURANT,
      "Restaurant et établissement avec service complet (sans terrasse) -Établissements avec permis alcoolinclus pub café et brasserie": cte.FULL_SERVICE_RESTAURANT,
      "Restaurant et établissement avec service restreint ( commande au comptoir ou par téléphone)": cte.QUICK_SERVICE_RESTAURANT,
      "Restaurant et établissement offrant des repas à libre-service (cafétéria cantine)": cte.QUICK_SERVICE_RESTAURANT,
      "Rue et avenue pour l'accès local": cte.NON_HEATED,
      "Ruelle": cte.NON_HEATED,
      "Récupération et triage de matières polluantes et toxiques": cte.WAREHOUSE,
      "Récupération et triage de métaux": cte.WAREHOUSE,
      "Réparation et entretien des avions": cte.WAREHOUSE,
      "Réserve pour la protection de la faune": cte.NON_HEATED,
      "Réservoir d'eau (installation d'emmagasinage de l'eau par retenue et réservoirs)": cte.NON_HEATED,
      "Résidence de tourismeappartement maison ou chalet (meublé et équipé pour repas)": cte.SMALL_HOTEL,
      "Salle d'exposition": cte.EVENT_LOCATION,
      "Salle et terrain de squash de racquetball et de tennis": cte.SPORTS_LOCATION,
      "Salle ou salon de quilles": cte.NON_HEATED,
      "Salon de beauté (maquillagemanucureetc..)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Salon de coiffure": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Salon funéraire": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Service bancaire (dépôts et prêtsincluant banque à charte)": cte.OFFICE_AND_ADMINISTRATION,
      "Service d'ambulance": cte.WAREHOUSE,
      "Service d'architecture": cte.OFFICE_AND_ADMINISTRATION,
      "Service d'assainissement de l'environnement": cte.WAREHOUSE,
      "Service d'emballage et de protection de marchandises": cte.WAREHOUSE,
      "Service d'envoi de marchandises": cte.WAREHOUSE,
      "Service d'hébergement des données (sites Web diffusion audio et vidéo en continu services d'application)": cte.WAREHOUSE,
      "Service d'hôpital (inclus hôpitaux psychiatriques)": cte.HOSPITAL,
      "Service d'optométrie": cte.OUT_PATIENT_HEALTH_CARE,
      "Service de buanderie de nettoyage à sec et de teinture (sauf les tapis)": cte.WAREHOUSE,
      "Service de comptabilitéde vérification et de tenue de livre": cte.OFFICE_AND_ADMINISTRATION,
      "Service de construction de routesde rues et de pontsde trottoirs et de pistes (entrepreneur général)": cte.WAREHOUSE,
      "Service de construction non résidentiellecommerciale et institutionnelle (entrepreneur général)": cte.WAREHOUSE,
      "Service de construction non résidentielleindustrielle (entrepreneur général)": cte.WAREHOUSE,
      "Service de construction résidentielle (entrepreneur)": cte.WAREHOUSE,
      "Service de débosselage et de peinture d'automobiles": cte.WAREHOUSE,
      "Service de garderie (prématernelle moins de 50 % de poupons)": cte.PRIMARY_SCHOOL,
      "Service de génie": cte.OFFICE_AND_ADMINISTRATION,
      "Service de holding et d'investissement et de fiducie": cte.OFFICE_AND_ADMINISTRATION,
      "Service de laboratoire dentaire": cte.OUT_PATIENT_HEALTH_CARE,
      "Service de laboratoire médical": cte.OUT_PATIENT_HEALTH_CARE,
      "Service de lavage d'automobiles": cte.WAREHOUSE,
      "Service de limousine": cte.WAREHOUSE,
      "Service de lingerie et de buanderie industrielle": cte.WAREHOUSE,
      "Service de location d'automobiles": cte.WAREHOUSE,
      "Service de location d'outils ou d'équipements": cte.WAREHOUSE,
      "Service de location d'équipements": cte.WAREHOUSE,
      "Service de location de boites postales (sauf le publipostage) et centre de courrier privé": cte.OFFICE_AND_ADMINISTRATION,
      "Service de location de camions de remorques utilitaires et de véhicules de plaisance": cte.WAREHOUSE,
      "Service de maçonnerie (entrepreneur spécialisé)": cte.INDUSTRY,
      "Service de messagers": cte.MEDIUM_OFFICE,
      "Service de notaires": cte.MEDIUM_OFFICE,
      "Service de paysagement ou de déneigement": cte.WAREHOUSE,
      "Service de petite menuiserie et de finition (entrepreneur spécialisé)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Service de plomberie de chauffagede climatisation et de ventilation (entrepreneur spécialisé)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Service de police fédérale et activités connexes": cte.OFFICE_AND_ADMINISTRATION,
      "Service de police municipale et activités connexes": cte.OFFICE_AND_ADMINISTRATION,
      "Service de pose de portesde fenêtres et de panneaux de verre": cte.WAREHOUSE,
      "Service de publicité en général": cte.MEDIUM_OFFICE,
      "Service de recherche de développement et d'essais": cte.SECONDARY_SCHOOL,
      "Service de remplacement de pièces et d'accessoires d'automobiles (amortisseurs silencieux toits ouvrants glacespare-brises...)": cte.WAREHOUSE,
      "Service de revêtement en asphalte et en bitume": cte.WAREHOUSE,
      "Service de réparation d'automobiles (garage) sans pompes à essence(5531)": cte.WAREHOUSE,
      "Service de réparation d'autres véhicules légers": cte.WAREHOUSE,
      "Service de réparation de véhicules légers motorisés (motocyclettemotoneige véhicule tout terrain)": cte.WAREHOUSE,
      "Service de réparation et d'entretien de machines et de matériel d'usage commercial et industriel": cte.WAREHOUSE,
      "Service de réparation et d'entretien de matériel informatique": cte.COMMERCIAL,
      "Service de réparation et d'entretien de systèmes de plomberieschauffageventilation et climatisation.(entrepreneur spécialisé)": cte.WAREHOUSE,
      "Service de réparation et d'entretien de véhicules lourds": cte.WAREHOUSE,
      "Service de réparation et de rembourrage de meubles": cte.WAREHOUSE,
      "Service de soudure": cte.WAREHOUSE,
      "Service de toilettage pour animaux domestiques": cte.OUT_PATIENT_HEALTH_CARE,
      "Service de traitement pour automobiles (antirouilleetc.)": cte.WAREHOUSE,
      "Service de travaux d'électricité et installation de câblage (entrepreneur spécialisé)": cte.WAREHOUSE,
      "Service de travaux de toiture (entrepreneur spécialisé)": cte.WAREHOUSE,
      "Service de télécommunication sans fil (appareil mobile sauf par Internet)": cte.WAREHOUSE,
      "Service de vétérinaires (animaux domestiques)": cte.OUT_PATIENT_HEALTH_CARE,
      "Service de vétérinaires et d'hôpital pour animaux de ferme": cte.OUT_PATIENT_HEALTH_CARE,
      "Service dentaire (inclus chirurgie et hygiène)": cte.OUT_PATIENT_HEALTH_CARE,
      "Service en santé mentale (cabinet) (comprend tous services professionnelspsychiatre psychologuepsychanalyste)": cte.OUT_PATIENT_HEALTH_CARE,
      "Service en travaux de fondation et de structures en béton (entrepreneur spécialisé)": cte.WAREHOUSE,
      "Service informatique (location ou utilisation partagée services auxiliaires programmation planification et analyse de système)": cte.MEDIUM_OFFICE,
      "Service médical (cabinet de médecins et chirurgiens spécialisés)": cte.OUT_PATIENT_HEALTH_CARE,
      "Service photographique (incluant les services commerciaux)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Service pour l'entretien ménager": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Service éducationnel et de recherche scientifique": cte.SECONDARY_SCHOOL,
      "Services spécialisés reliés à l'activité bancaire": cte.OFFICE_AND_ADMINISTRATION,
      "Stade": cte.SPORTS_LOCATION,
      "Station de contrôle de la pression de l'eau": cte.WAREHOUSE,
      "Station de contrôle de la pression des eaux usées": cte.WAREHOUSE,
      "Station de métro": cte.WAREHOUSE,
      "Station libre-serviceou avec service et dépanneur sans réparation de véhicules automobiles": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Station libre-serviceou avec service sans réparation de véhicules automobiles": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Station-service avec réparation de véhicules automobiles": cte.WAREHOUSE,
      "Stationnement extérieur (condo non résidentiel)": cte.NON_HEATED,
      "Stationnement extérieur (condo)": cte.NON_HEATED,
      "Stationnement intérieur ( condo non résidentiel)": cte.WAREHOUSE,
      "Stationnement intérieur (condo)": cte.WAREHOUSE,
      "Studio d'enregistrement du son (disque cassette et disque compact)": cte.MEDIUM_OFFICE,
      "Studio de production de filmsde vidéos ou de publicités (ne comprends pas le laboratoire de production)": cte.MEDIUM_OFFICE,
      "Studio de télévision (sans public)": cte.MEDIUM_OFFICE,
      "Syndicat et organisation similaire": cte.MEDIUM_OFFICE,
      "Terminus maritime (passagers) incluant les gares de traversiers": cte.WAREHOUSE,
      "Terrain de golf (avec chalet et autres aménagements sportifs)": cte.NON_HEATED,
      "Terrain de sport (jeux et pistes pour compétitions et sportgradins)": cte.NON_HEATED,
      "Terrain de stationnement pour automobiles": cte.WAREHOUSE,
      "Terrains de stationnement pour véhicules lourds": cte.WAREHOUSE,
      "Théâtre": cte.EVENT_LOCATION,
      "Tour de relais (micro-ondes)": cte.NON_HEATED,
      "Tous les autres services d'information": cte.NON_HEATED,
      "Transport et gestion d'électricité en bloc": cte.NON_HEATED,
      "Transport et gestion du gaz par canalisation": cte.NON_HEATED,
      "Université": cte.SECONDARY_SCHOOL,
      "Usine de traitement des eaux (filtration)": cte.INDUSTRY,
      "Usine de traitement des eaux usées (épuration)": cte.INDUSTRY,
      "Vente au détail (fleuriste)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail d'accessoires pour femmes": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail d'animaux de maison (animalerie)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail d'antiquités (sauf le marché aux puces)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail d'appareils orthopédiques et articles spécialisés de santé": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail d'articles de sport": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail d'articles d'accessoires d'aménagement paysager et de jardin": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail d'instruments et de matériel médical": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail d'équipements de ferme": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail d'équipements de plomberie de chauffagede ventilationde climatisation et de foyer": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail d'équipements et d'accessoires de chasse et pêche": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de bicyclettes": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de boissons alcoolisées": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de chaussures": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de fruits et de légumes": cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
      "Vente au détail de la viande": cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
      "Vente au détail de livres et de papeterie": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de marchandises en général (sauf les marchés aux puces)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de matériaux de construction": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de matériaux de construction (cour à bois)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de matériel électrique": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de meubles": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de motocyclettes de motoneiges et de leurs accessoires": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de médicaments et d'articles divers (pharmacie)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de peinturede verre et de papier tenture": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de piscinesde spas et leurs accessoires": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de pièces de véhicules automobiles et d'accessoires usagés": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de pneus de batteries et d'accessoires": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de produits d'épicerie (avec boucherie)": cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
      "Vente au détail de produits d'épicerie (sans boucherie)": cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
      "Vente au détail de produits de la boulangerie et de la pâtisserie (manufacturés sur place en totalité ou non)": cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
      "Vente au détail de quincaillerie": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de radiosde téléviseurssystèmes de son et appareils électroniques": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de revêtements de planchers et de murs (bois franc plancher flottant carreaux céramiques tapisserie)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de serruresde clés et d'accessoires": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de véhicules automobiles neufs et usagés": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de véhicules automobiles usagés seulement": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de vêtement prêt-à-porter pour femmes": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de vêtements et d'accessoires pour hommes": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de vêtements et d'articles usagésfriperies (sauf le marché aux puces)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de vêtements unisexes": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail du cafédu théd'épices et d'aromates": cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
      "Vente au détailclubs de gros et hypermarchés (entrepôt-club)": cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
      "Vente au détailfournitures pour la maison et l'auto": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détailmagasin à rayons": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente en gros d'ameublements de matériels de bureau et de magasin": cte.STRIP_MALL,
      "Vente en gros d'appareils et d'équipements de plomberie et de chauffage": cte.STRIP_MALL,
      "Vente en gros d'appareils et d'équipements électriques de fils et de matériel électronique de construction": cte.STRIP_MALL,
      "Vente en gros d'appareils électriquesde téléviseurs et de radios": cte.STRIP_MALL,
      "Vente en gros d'automobiles et autres véhicules automobiles neufs ou d'occasions incluent VR)": cte.STRIP_MALL,
      "Vente en gros d'autres appareils ou matériels électriques et électroniques": cte.STRIP_MALL,
      "Vente en gros d'autres médicaments de produits chimiques et de produits connexes": cte.STRIP_MALL,
      "Vente en gros d'autres pièces d'équipement ou de machinerie (incluant machinerie lourde)": cte.STRIP_MALL,
      "Vente en gros d'autres produits reliés à l'épicerie": cte.STRIP_MALL,
      "Vente en gros d'équipements et de pièces de machinerie commercialeindustrielle ou agricole (incluant machinerie lourde)": cte.STRIP_MALL,
      "Vente en gros d'équipements et de pièces pour la réfrigération ventilation la climatisation et le chauffage (système combiné)": cte.STRIP_MALL,
      "Vente en gros d'équipements et de pièces pour les entreprises de services": cte.STRIP_MALL,
      "Vente en gros de bois et de matériaux de construction": cte.STRIP_MALL,
      "Vente en gros de chaussures": cte.STRIP_MALL,
      "Vente en gros de fruits et de légumes frais": cte.STRIP_MALL,
      "Vente en gros de médicaments et de produits médicamenteux": cte.STRIP_MALL,
      "Vente en gros de pièces et d'accessoires neufs pour véhicules automobiles": cte.STRIP_MALL,
      "Vente en gros de pièces et d'équipements électroniques": cte.STRIP_MALL,
      "Vente en gros de pneus et de chambres à air": cte.STRIP_MALL,
      "Vente en gros de poissons et de fruits de mer": cte.STRIP_MALL,
      "Vente en gros de produits de beauté": cte.STRIP_MALL,
      "Vente en gros de produits de boulangerie et de pâtisserie": cte.STRIP_MALL,
      "Vente en gros de produits laitiers": cte.STRIP_MALL,
      "Vente en gros de quincaillerie": cte.STRIP_MALL,
      "Vente en gros de tissus et de textiles": cte.STRIP_MALL,
      "Vente en gros de viandes et de produits de la viande": cte.STRIP_MALL,
      "Vente en gros de vêtements de lingerie de bas et d'accessoires": cte.STRIP_MALL,
      "Vente en gros pour l'épicerie en général": cte.STRIP_MALL,
      "École commerciale et de secrétariat (non intégrée aux polyvalentes)": cte.SECONDARY_SCHOOL,
      "École de beaux-arts et de musique (exclus arts publicitaires arts graphiques et photographie publicitaire)": cte.SECONDARY_SCHOOL,
      "École de danse": cte.SECONDARY_SCHOOL,
      "École de métiers (non intégrée aux polyvalentes)": cte.SECONDARY_SCHOOL,
      "École maternelle": cte.SECONDARY_SCHOOL,
      "École polyvalente": cte.SECONDARY_SCHOOL,
      "École secondaire": cte.SECONDARY_SCHOOL,
      "École à caractère familial (exploité par une personne physique dans sa résidence moins de 15 élèves)": cte.SECONDARY_SCHOOL,
      "École élémentaire": cte.SECONDARY_SCHOOL,
      "École élémentaire et secondaire": cte.SECONDARY_SCHOOL,
      "Église synagogue mosquée et temple": cte.EVENT_LOCATION,
      "Établissement avec salle de réception ou de banquet": cte.FULL_SERVICE_RESTAURANT,
      "Établissement avec service de boissons alcoolisées (Bar)": cte.QUICK_SERVICE_RESTAURANT,
      "Établissement dont l'activité principale est la danse (discothèque avec service alcool boite de nuit) sans alcool code 7397": cte.QUICK_SERVICE_RESTAURANT,
      "Mausolée": cte.NON_HEATED,
      "Auberge ou gîte touristique (Hôtel à caractère familial, d'au plus 3 étages en hauteur de bâtiment)": cte.HOTEL,
      "Service de garderie (prématernelle, moins de 50 % de poupons)": cte.PRIMARY_SCHOOL,
      "Église, synagogue, mosquée et temple": cte.CONVENTION_CENTER,

      "Station libre-service, ou avec service et dépanneur sans réparation de véhicules automobiles": cte.WAREHOUSE,
      "Service de buanderie, de nettoyage à sec et de teinture (sauf les tapis)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Service bancaire (dépôts et prêts, incluant banque à charte)": cte.COMMERCIAL,
      "Restaurant et établissement avec service complet (avec terrasse) - Établissements avec permis alcool, inclus pub,café et brasserie": cte.FULL_SERVICE_RESTAURANT,
      "Restaurant et établissement avec service complet (sans terrasse) -Établissements avec permis alcool, inclus pub,café et brasserie": cte.FULL_SERVICE_RESTAURANT,
      "Autres services immobiliers, financiers et d'assurance": cte.COMMERCIAL,
      "Résidence de tourisme, appartement, maison ou chalet (meublé et équipé pour repas)": cte.RESIDENTIAL,
      "Salon de beauté (maquillage, manucure, etc..)": cte.COMMERCIAL,
      "Service de buanderie et de nettoyage à sec (libre-service)": cte.COMMERCIAL,
      "Association civique, sociale et fraternelle": cte.COMMERCIAL,
      "Vente en gros d'automobiles et autres véhicules automobiles neufs ou d'occasions, incluent VR)": cte.WAREHOUSE,
      "Autres services de soutien aux entreprises": cte.COMMERCIAL,
      "Vente au détail de vêtements pour toute la famille": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de bijoux": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Service d'avocats": cte.MEDIUM_OFFICE,
      "Service informatique (location ou utilisation partagée, services auxiliaires programmation planification et analyse de système)": cte.MEDIUM_OFFICE,
      "Centre d'entraide et de ressources communautaires (inclus ressources d'hébergement, de meubles et d'alimentation)": cte.COMMERCIAL,
      "Service de comptabilité, de vérification et de tenue de livre": cte.MEDIUM_OFFICE,
      "Autres transports, communications et services publics (infrastructure)": cte.WAREHOUSE,
      "Autres institutions de formation spécialisée (inclus écoles de langues, de couture, d'arts martiaux, de combats et autres)": cte.SECONDARY_SCHOOL,
      "Chemin de fer (sauf train touristique, aiguillage et cour de triage)": cte.WAREHOUSE,
      "Service d'agence de placement": cte.COMMERCIAL,
      "Espace de plancher inoccupé dont l'usage serait la vente au détail": cte.NON_HEATED,
      "Industrie de produits en plastique stratifié, sous pression ou renforcé": cte.INDUSTRY,
      "Studio de production de films, de vidéos ou de publicités (ne comprends pas le laboratoire de production)": cte.MOTION_PICTURE_THEATRE,
      "Industrie de la fabrication de supports d'enregistrement, de la reproduction du son et des instruments de musique": cte.INDUSTRY,
      "Vente au détail du café, du thé, d'épices et d'aromates": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Station libre-service, ou avec service sans réparation de véhicules automobiles": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de produits de la boulangerie et de la pâtisserie (non manufacturés sur place)": cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
      "Vente au détail de serrures, de clés et d'accessoires": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Autres activités de vente au détail de vêtements, comme les accessoires": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente en gros d'équipements et de pièces pour la réfrigération, ventilation, la climatisation et le chauffage (système combiné)": cte.WAREHOUSE,
      "Service de réparation de véhicules légers motorisés (motocyclette, motoneige, véhicule tout terrain)": cte.WAREHOUSE,
      "Service de location de camions, de remorques utilitaires et de véhicules de plaisance": cte.WAREHOUSE,
      "Vente au détail d'autres articles de ferme": cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
      "Salon d'esthétique (épilation, traitement de la peau)": cte.COMMERCIAL,
      "Service de location de films, de jeux vidéos et de matériel audiovisuel": cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
      "Vente en gros de vêtements, de lingerie, de bas et d'accessoires": cte.WAREHOUSE,
      "Service d'entretien de chaussures et d'articles de cuir (cordonnerie)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de lingerie pour enfants": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail, magasin à rayons": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de vêtements et d'articles usagés, friperies (sauf le marché aux puces)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Entreposage du mobilier et d'appareils ménagers, incluant les mini-entrepôts": cte.WAREHOUSE,
      "Service de construction non résidentielle, industrielle (entrepreneur général)": cte.WAREHOUSE,
      "Immeuble à temps partagé («time share») Propriété ou copropriété ou groupe d'usufruitier ont chacun droit de jouissance, périodique et successif.": cte.COMMERCIAL,
      "Autres activités récréatives": cte.EVENT_LOCATION,
      "Industrie de produits de boulangerie commerciale, de produits de boulangerie congelés et de pâtisseries": cte.INDUSTRY,
      "Studio d'enregistrement du son (disque, cassette et disque compact)": cte.COMMERCIAL,
      "Autres services de soins paramédicaux": cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
      "Studio de radiodiffusion (sans public)": cte.COMMERCIAL,
      "Autres présentations d'objets ou d'animaux": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Autres étendues d'eau": cte.WAREHOUSE,
      "Maison d'agents, de courtiers et de services d'administration des biens-fonds": cte.MEDIUM_OFFICE,
      "Industrie de l'édition du livre": cte.INDUSTRY,
      "Vente au détail ou location d'articles,d'accessoires de scène et de costumes": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "École de beaux-arts et de musique (exclus arts publicitaires, arts graphiques et photographie publicitaire)": cte.SECONDARY_SCHOOL,
      "Vente au détail de peinture, de verre et de papier tenture": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Agence et courtier d'assurances": cte.MEDIUM_OFFICE,
      "Centre de santé (inclus saunas, spas et bains thérapeutiques ou turcs)": cte.HEALTH_CARE,
      "Établissement dont l'activité principale est la danse (discothèque avec service alcool, boite de nuit) sans alcool code 7397": cte.QUICK_SERVICE_RESTAURANT,
      "Vente au détail de pneus, de batteries et d'accessoires": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Salle et terrain de squash, de racquetball et de tennis": cte.WAREHOUSE,
      "Service de télécommunication sans fil (appareil mobile, sauf par Internet)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Centre de recherche en science physique et chimique (inclus science optique)": cte.UNIVERSITY,
      "Association, union ou coop d'épargne et de prêt (inclus caisses populaires locales)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Industrie du meuble et d'articles d'ameublement pour hôtels, restaurants et institutions": cte.INDUSTRY,
      "Service d'affichage à l'extérieur": cte.NON_HEATED,
      "Service de traitement pour automobiles (antirouille, etc.)": cte.INDUSTRY,
      "Vente au détail de revêtements de planchers et de murs (bois franc, plancher flottant, carreaux céramiques, tapisserie)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de livres et de journaux": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Autres industries du laminage, du moulage et de l'extrusion de métaux non-ferreux": cte.INDUSTRY,
      "Vente au détail d'appareils ménagers": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Service de plomberie, de chauffage, de climatisation et de ventilation (entrepreneur spécialisé)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de radios, de téléviseurs, systèmes de son et appareils électroniques": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail, clubs de gros et hypermarchés (entrepôt-club)": cte.STRIP_MALL,
      "Conserverie, marinage, saumurage et séchage de fruits et de légumes": cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
      "Industrie de l'étirage, de l'extrusion et alliage de l'aluminium, fabriqué à partir d'aluminium acheté": cte.INDUSTRY,
      "Restaurant et établissement offrant des repas à libre-service (cafétéria, cantine)": cte.QUICK_SERVICE_RESTAURANT,
      "Pressage de vêtements": cte.COMMERCIAL,
      "Vente au détail d'articles, d'accessoires d'aménagement paysager et de jardin": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Planétarium": cte.MUSEUM,
      "Industrie de ventilateurs, de soufflantes et de purificateurs d'air industriels et commerciaux": cte.INDUSTRY,
      "Service de pose de portes, de fenêtres et de panneaux de verre": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Association de personnes exerçant une même profession ou une même activité": cte.MEDIUM_OFFICE,
      "Service de remplacement de pièces et d'accessoires d'automobiles (amortisseurs, silencieux,toits ouvrants, glaces, pare-brises...)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Service d'hébergement des données (sites Web, diffusion audio et vidéo en continu, services d'application)": cte.DATACENTER,
      "Vente au détail de produits naturels et aliments de régime": cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
      "Autres activités de vente au détail reliées aux automobiles, aux embarcations, aux avions et à leurs accessoires": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Autres lieux d'amusement": cte.EVENT_LOCATION,
      "Vente en gros d'appareils et d'équipements électriques, de fils et de matériel électronique de construction": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Industrie de peinture, de teinture et de vernis": cte.INDUSTRY,
      "Autres industries de l'impression et de l'édition (combinées)": cte.INDUSTRY,
      "Vente en gros d'équipements et de pièces de machinerie commerciale,industrielle ou agricole (incluant machinerie lourde)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Industrie de lingerie, de vêtements de détente et de vêtements de nuit pour femmes et filles": cte.INDUSTRY,
      "Vente au détail d'équipements de plomberie, de chauffage, de ventilation, de climatisation et de foyer": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Autres industries de produits minéraux non métalliques": cte.INDUSTRY,
      "Entreprise d'excavation, de nivellement, de défrichage et installation de fosses septiques": cte.NON_HEATED,
      "Vente au détail, fournitures pour la maison et l'auto": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Industrie d'armoires, de placards de cuisine et de coiffeuses de salle de bains en bois": cte.INDUSTRY,
      "Terrain de sport (jeux et pistes pour compétitions et sport, gradins)": cte.SPORTS_LOCATION,
      "Vente au détail de piscines, de spas et leurs accessoires": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Service en santé mentale (cabinet) (comprend tous services professionnels, psychiatre,psychologue, psychanalyste)": cte.HEALTH_CARE,
      "Service de construction de routes, de rues et de ponts,de trottoirs et de pistes (entrepreneur général)": cte.NON_HEATED,
      "Vente en gros d'appareils électriques, de téléviseurs et de radios": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Vente au détail de motocyclettes, de motoneiges et de leurs accessoires": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Industrie du meuble rembourré résidentiel (rembourrage, service de réparation, voir 6423)": cte.INDUSTRY,
      "Vente au détail de jouets et d'articles de jeux": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Industrie du laminage, de l'étirage et de l'extrusion du cuivre et de ses alliages": cte.INDUSTRY,
      "Archives (incluant cinémathèque, vidéothèque)": cte.WAREHOUSE_REFRIGERATED,
      "Service de recherche, de développement et d'essais": cte.UNIVERSITY,
      "Industrie de tapis, carpettes et moquettes": cte.INDUSTRY,
      "Industrie du revêtement métallique, sur commande": cte.INDUSTRY,
      "Service de réparation et d'entretien de systèmes de plomberies,chauffage, ventilation et climatisation.(entrepreneur spécialisé)": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Salle de réunions, centre de conférence et congrès": cte.CONVENTION_CENTER,
      "Vente en gros d'autres médicaments, de produits chimiques et de produits connexes": cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
      "Vente au détail d'armoires de coiffeuses et de meubles d'appoint": cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
      "Industrie de chaudières, d'échangeurs de chaleur et de plaques métalliques": cte.INDUSTRY,
      "Industrie du clichage, de la composition de la reliure et de la lithographie": cte.INDUSTRY,
      "Service de construction non résidentielle, commerciale et institutionnelle (entrepreneur général)": cte.MEDIUM_OFFICE,
      "Centre d'entreposage de produits pétroliers (pétrole brut, gaz pétrole liquéfié, mazout domestique et autres produits raffinés)": cte.WAREHOUSE,
      "École à caractère familial (exploité par une personne physique dans sa résidence, moins de 15 élèves)": cte.PRIMARY_SCHOOL
    }

  @property
  def dictionary(self) -> dict:
    return self._dictionary



"""
Bus system module
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""

from typing import List
from hub.city_model_structure.city_object import CityObject
from hub.city_model_structure.attributes.polygon import Polygon
from hub.city_model_structure.transport.bus_network import BusNetwork
from hub.city_model_structure.transport.bus_node import BusNode
from hub.city_model_structure.transport.bus import Bus


class BusSystem(CityObject):
  """
  BusSystem(CityObject) class
  """
  def __init__(self, name, surfaces):
    super().__init__(name, surfaces)
    self._bus_routes = None
    self._bus_network = None
    self._buses = None
    self._restricted_polygons = None

  @property
  def bus_routes(self) -> List[BusNode]:
    """
    Add explanation here
    :return: [BusNode]
    """
    return self._bus_routes

  @property
  def bus_network(self) -> BusNetwork:
    """
    Add explanation here
    :return: BusNetwork
    """
    return self._bus_network

  @property
  def buses(self) -> List[Bus]:
    """
    Add explanation here
    :return: [Bus]
    """
    return self._buses

  @property
  def restricted_polygons(self) -> List[Polygon]:
    """
    Add explanation here
    :return: [Polygon]
    """
    return self._restricted_polygons



"""
WeatherFactory retrieve the specific weather module for the given source format
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""
from pathlib import Path
from hub.imports.weather.xls_weather_parameters import XlsWeatherParameters
from hub.imports.weather.epw_weather_parameters import EpwWeatherParameters
from hub.hub_logger import logger
from hub.helpers.utils import validate_import_export_type


class WeatherFactory:
  """
  WeatherFactory class
  """

  def __init__(self, handler, city, base_path=None, file_name=None):
    if base_path is None:
      base_path = Path(Path(__file__).parent.parent / 'data/weather')
    self._handler = '_' + handler.lower().replace(' ', '_')
    class_funcs = validate_import_export_type(WeatherFactory)
    if self._handler not in class_funcs:
      err_msg = f"Wrong import type. Valid functions include {class_funcs}"
      logger.error(err_msg)
      raise Exception(err_msg)
    self._city = city
    self._base_path = base_path
    self._file_name = file_name

  def _epw(self):
    """
    Enrich the city with energy plus weather file
    """
    # EnergyPlus Weather
    # to download files: https://energyplus.net/weather
    # description of the format: https://energyplus.net/sites/default/files/pdfs_v8.3.0/AuxiliaryPrograms.pdf
    _path = Path(self._base_path / 'epw').resolve()
    return EpwWeatherParameters(self._city, _path, self._file_name)

  def _xls(self):
    """
    Enrich the city with ISO_52016_1_BESTEST_ClimData_2016.08.24 weather file
    """
    name = 'ISO_52016_1_BESTEST_ClimData_2016.08.24'
    return XlsWeatherParameters(self._city, self._base_path, name)

  def enrich(self):
    """
    Enrich the city given to the class using the given weather handler
    :return: None
    """
    getattr(self, self._handler, lambda: None)()



"""
LifeCycleAssessment retrieve the specific Life Cycle Assessment module for the given region
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Atiya atiya.atiya@mail.concordia.ca
"""

class Vehicle:
  """
  Vehicle class
  """

  def __init__(self, vehicle_id, name, fuel_consumption_rate, fuel_consumption_unit, carbon_emission_factor,
               carbon_emission_factor_unit):
    self._vehicle_id = vehicle_id
    self._name = name
    self._fuel_consumption_rate = fuel_consumption_rate
    self._fuel_consumption_unit = fuel_consumption_unit
    self._carbon_emission_factor = carbon_emission_factor
    self._carbon_emission_factor_unit = carbon_emission_factor_unit

  @property
  def id(self) -> int:
    """
    Get vehicle id
    :return: int
    """
    return self._vehicle_id

  @property
  def name(self) -> str:
    """
    Get vehicle name
    :return: str
    """
    return self._name

  @property
  def fuel_consumption_rate(self) -> float:
    """
    Get vehicle fuel consumption rate
    :return: float
    """
    return self._fuel_consumption_rate

  @property
  def fuel_consumption_unit(self) -> str:
    """
    Get fuel consumption unit
    :return: str
    """
    return self._fuel_consumption_unit

  @property
  def carbon_emission_factor(self) -> float:
    """
    Get vehicle carbon emission factor
    :return: float
    """
    return self._carbon_emission_factor

  @property
  def carbon_emission_factor_unit(self) -> str:
    """
    Get carbon emission units
    :return: str
    """
    return self._carbon_emission_factor_unit



"""
ConstructionFactory (before PhysicsFactory) retrieve the specific construction module for the given region
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Atiya atiya.atiya@mail.concordia.ca
"""

class Fuel:
  def __init__(self, fuel_id, name, carbon_emission_factor, unit):
    self._fuel_id = fuel_id
    self._name = name
    self._carbon_emission_factor = carbon_emission_factor
    self._unit = unit

  @property
  def id(self) -> int:
    """
    Get fuel id
    :return: int
    """
    return self._fuel_id

  @property
  def name(self) -> str:
    """
    Get fuel name
    :return: str
    """
    return self._name

  @property
  def carbon_emission_factor(self) -> float:
    """
    Get fuel carbon emission factor
    :return: float
    """
    return self._carbon_emission_factor

  @property
  def unit(self) -> str:
    """
    Get fuel units
    :return: str
    """
    return self._unit



"""
LifeCycleAssessment retrieve the specific Life Cycle Assessment module for the given region
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Atiya atiya.atiya@mail.concordia.ca
"""


class Machine:
  """
  Machine class
  """

  def __init__(self, machine_id, name, work_efficiency, work_efficiency_unit, energy_consumption_rate,
               energy_consumption_unit, carbon_emission_factor, carbon_emission_unit):
    self._machine_id = machine_id
    self._name = name
    self._work_efficiency = work_efficiency
    self._work_efficiency_unit = work_efficiency_unit
    self._energy_consumption_rate = energy_consumption_rate
    self._energy_consumption_unit = energy_consumption_unit
    self._carbon_emission_factor = carbon_emission_factor
    self._carbon_emission_unit = carbon_emission_unit

  @property
  def id(self) -> int:
    """
    Get machine id
    :return: int
    """
    return self._machine_id

  @property
  def name(self) -> str:
    """
    Get machine name
    :return: str
    """
    return self._name

  @property
  def work_efficiency(self) -> float:
    """
    Get machine work efficiency
    :return: float
    """
    return self._work_efficiency

  @property
  def work_efficiency_unit(self) -> str:
    """
    Get machine work efficiency unit
    :return: str
    """
    return self._work_efficiency_unit

  @property
  def energy_consumption_rate(self) -> float:
    """
    Get energy consumption rate
    :return: float
    """
    return self._energy_consumption_rate

  @property
  def energy_consumption_unit(self) -> str:
    """
    Get energy consumption unit
    :return: str
    """
    return self._energy_consumption_unit

  @property
  def carbon_emission_factor(self) -> float:
    """
    Get carbon emission factor
    :return: float
    """
    return self._carbon_emission_factor

  @property
  def carbon_emission_unit(self) -> str:
    """
    Get carbon emission unit
    :return: str
    """
    return self._carbon_emission_unit



"""
Dictionaries module for Alkis function to hub function
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez Guillermo.GutierrezMorote@concordia.ca
"""


import hub.helpers.constants as cte

class AlkisFunctionToHubFunction:

  def __init__(self):
    self._dictionary = {"1000": cte.RESIDENTIAL,
                        "1010": "tenement",
                        "1020": "hostel",
                        "1030": "residential- and administration building",
                        "1040": "residential- and office building",
                        "1050": "residential- and business building",
                        "1060": "residential- and plant building",
                        "1070": "agrarian- and forestry building",
                        "1080": "residential- and commercial building",
                        "1090": "forester's lodge",
                        "1100": "holiday house",
                        "1110": "summer house",
                        "1120": "office building",
                        "1130": "credit institution",
                        "1140": "insurance",
                        "1150": "business building",
                        "1160": "department store",
                        "1170": "shopping centre",
                        "1180": "kiosk",
                        "1190": "pharmacy",
                        "1200": "pavilion",
                        "1210": cte.HOTEL,
                        "1220": "youth hostel",
                        "1230": "campsite building",
                        "1240": "restaurant",
                        "1250": "cantine",
                        "1260": "recreational site",
                        "1270": "function room",
                        "1280": "cinema",
                        "1290": "bowling alley",
                        "1300": "casino",
                        "1310": "industrial building",
                        "1320": "factory",
                        "1330": cte.WORKSHOP,
                        "1340": "petrol / gas station",
                        "1350": "washing plant",
                        "1360": "cold store",
                        "1370": "depot",
                        "1380": "building for research purposes",
                        "1390": "quarry",
                        "1400": "salt works",
                        "1410": "miscellaneous industrial building",
                        "1420": "mill",
                        "1430": "windmill",
                        "1440": "water mill",
                        "1450": "bucket elevator",
                        "1460": "weather station",
                        "1470": "traffic assets office",
                        "1480": "street maintenance",
                        "1490": "waiting hall",
                        "1500": "signal control box",
                        "1510": "engine shed",
                        "1520": "signal box or stop signal",
                        "1530": "plant building for air traffic",
                        "1540": "hangar",
                        "1550": "plant building for shipping",
                        "1560": "shipyard",
                        "1570": "dock",
                        "1580": "plant building for canal lock",
                        "1590": "boathouse",
                        "1600": "plant building for cablecar",
                        "1610": "multi-storey car park",
                        "1620": "parking level",
                        "1630": "garage",
                        "1640": "vehicle hall",
                        "1650": "underground garage",
                        "1660": "building for supply",
                        "1670": "waterworks",
                        "1680": "pump station",
                        "1690": "water basin",
                        "1700": "electric power station",
                        "1710": "transformer station",
                        "1720": "converter",
                        "1730": "reactor",
                        "1740": "turbine house",
                        "1750": "boiler house",
                        "1760": "building for telecommunications",
                        "1770": "gas works",
                        "1780": "heat plant",
                        "1790": "pumping station",
                        "1800": "building for disposal",
                        "1810": "building for effluent disposal",
                        "1820": "building for filter plant",
                        "1830": "toilet",
                        "1840": "rubbish bunker",
                        "1850": "building for rubbish incineration",
                        "1860": "building for rubbish disposal",
                        "1870": "building for agrarian and forestry",
                        "1880": "barn",
                        "1890": "stall",
                        "1900": "equestrian hall",
                        "1910": "alpine cabin",
                        "1920": "hunting lodge",
                        "1930": "arboretum",
                        "1940": "glass house",
                        "1950": "moveable glass house",
                        "1960": "public building",
                        "1970": "administration building",
                        "1980": "parliament",
                        "1990": "guildhall",
                        "2000": "post office",
                        "2010": "customs office",
                        "2020": "court",
                        "2030": "embassy or consulate",
                        "2040": "district administration",
                        "2050": "district government",
                        "2060": "tax office",
                        "2070": "building for education and research",
                        "2080": "comprehensive school",
                        "2090": "vocational school",
                        "2100": "college or university",
                        "2110": "research establishment",
                        "2120": "building for cultural purposes",
                        "2130": "castle",
                        "2140": "theatre or opera",
                        "2150": "concert building",
                        "2160": cte.MUSEUM,
                        "2170": "broadcasting building",
                        "2180": "activity building",
                        "2190": cte.LIBRARY,
                        "2200": "fort",
                        "2210": "religious building",
                        "2220": "church",
                        "2230": "synagogue",
                        "2240": "chapel",
                        "2250": "community center",
                        "2260": "place of worship",
                        "2270": "mosque",
                        "2280": "temple",
                        "2290": "convent",
                        "2300": "building for health care",
                        "2310": cte.HOSPITAL,
                        "2320": "healing centre or care home",
                        "2330": "health centre or outpatients clinic",
                        "2340": "building for social purposes",
                        "2350": "youth centre",
                        "2360": "seniors centre",
                        "2370": "homeless shelter",
                        "2380": "kindergarten or nursery",
                        "2390": "asylum seekers home",
                        "2400": cte.POLICE_STATION,
                        "2410": cte.FIRE_STATION,
                        "2420": "barracks",
                        "2430": "bunker",
                        "2440": cte.PENITENTIARY,
                        "2450": "cemetery building",
                        "2460": "funeral parlor",
                        "2470": "crematorium",
                        "2480": "train station",
                        "2490": "airport building",
                        "2500": "building for underground station",
                        "2510": "building for tramway",
                        "2520": "building for bus station",
                        "2530": "shipping terminal",
                        "2540": "building for recuperation purposes",
                        "2550": "building for sport purposes",
                        "2560": "sports hall",
                        "2570": "building for sports field",
                        "2580": "swimming baths",
                        "2590": "indoor swimming pool",
                        "2600": "sanatorium",
                        "2610": "zoo building",
                        "2620": cte.GREEN_HOUSE,
                        "2630": "botanical show house",
                        "2640": "bothy",
                        "2650": "tourist information centre",
                        "2700": "others",
                        }

  @property
  def dictionary(self) -> dict:
    return self._dictionary



"""
Database setup
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Peter Yefi peteryefi@gmail.com
"""

from hub.persistence import Repository
from hub.persistence.models import Application
from hub.persistence.models import City
from hub.persistence.models import CityObject
from hub.persistence.models import User
from hub.persistence.models import UserRoles
from hub.persistence.models import SimulationResults
from hub.persistence.repositories import User as UserRepository
from hub.persistence.repositories import Application as ApplicationRepository
from hub.hub_logger import logger


class DBSetup:

  def __init__(self, db_name, app_env, dotenv_path, admin_password, application_uuid):
    """
    Creates database tables a default admin user and a default admin app with the given password and uuid
    :param db_name:
    :param app_env:
    :param dotenv_path:
    """
    repository = Repository(db_name=db_name, app_env=app_env, dotenv_path=dotenv_path)

    # Create the tables using the models
    Application.__table__.create(bind=repository.engine, checkfirst=True)
    User.__table__.create(bind=repository.engine, checkfirst=True)
    City.__table__.create(bind=repository.engine, checkfirst=True)
    CityObject.__table__.create(bind=repository.engine, checkfirst=True)
    SimulationResults.__table__.create(bind=repository.engine, checkfirst=True)

    self._user_repo = UserRepository(db_name=db_name, app_env=app_env, dotenv_path=dotenv_path)
    self._application_repo = ApplicationRepository(db_name=db_name, app_env=app_env, dotenv_path=dotenv_path)
    application_id = self._create_admin_app(self._application_repo, application_uuid)
    self._create_admin_user(self._user_repo, admin_password, application_id)

  @staticmethod
  def _create_admin_app(application_repo, application_uuid):
    name = 'AdminTool'
    description = 'Admin tool to control city persistence and to test the API v1.4'
    print('Creating default admin tool application...')
    application = application_repo.insert(name, description, application_uuid)

    if type(application) is dict:
      logger.info(application)
    else:
      msg = f'Created Admin tool with application_uuid: {application_uuid}'
      print(msg)
      logger.info(msg)
      return application.id

  @staticmethod
  def _create_admin_user(user_repo, admin_password, application_id):
    password = admin_password
    print('Creating default admin user...')
    user = user_repo.insert('Administrator', password, UserRoles.Admin, application_id)
    if type(user) is dict:
      logger.info(user)
    else:
      print(f'Created Admin user')
      logger.info(f'Created Admin user')



"""
TestCityMerge test and validate the merge of several cities into one
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez Guillermo.GutierrezMorote@concordia.ca
"""
from pathlib import Path
from unittest import TestCase

from hub.imports.geometry_factory import GeometryFactory


class TestCityMerge(TestCase):
  """
  Functional TestCityMerge
  """
  def setUp(self) -> None:
    """
    Test setup
    :return: None
    """
    self._example_path = (Path(__file__).parent / 'tests_data').resolve()

  def _get_citygml(self, file):
    file_path = (self._example_path / file).resolve()
    city = GeometryFactory('citygml', path=file_path).city
    self.assertIsNotNone(city, 'city is none')
    return city

  def test_merge(self):
    city_1 = self._get_citygml('one_building_in_kelowna.gml')
    city_2 = self._get_citygml('pluto_building.gml')
    city = city_1.merge(city_2)
    self.assertEqual(len(city_1.city_objects), 1, 'Wrong amount of city_objects found in city_1')
    self.assertEqual(len(city_2.city_objects), 1, 'Wrong amount of city_objects found in city_2')
    self.assertEqual(len(city.city_objects), 2, 'Wrong amount of city_objects found in city')



"""
Dictionaries module for hub usage to Comnet usage
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez Guillermo.GutierrezMorote@concordia.ca
"""

import hub.helpers.constants as cte


class HubUsageToComnetUsage:

  def __init__(self):
    self._dictionary = {
    cte.RESIDENTIAL: 'BA Multifamily',
    cte.SINGLE_FAMILY_HOUSE: 'BA Multifamily',
    cte.MULTI_FAMILY_HOUSE: 'BA Multifamily',
    cte.ROW_HOUSE: 'BA Multifamily',
    cte.MID_RISE_APARTMENT: 'BA Multifamily',
    cte.HIGH_RISE_APARTMENT: 'BA Multifamily',
    cte.OFFICE_AND_ADMINISTRATION: 'BA Office',
    cte.SMALL_OFFICE: 'BA Office',
    cte.MEDIUM_OFFICE: 'BA Office',
    cte.LARGE_OFFICE: 'BA Office',
    cte.COURTHOUSE: 'BA Courthouse',
    cte.FIRE_STATION: 'BA Fire Station',
    cte.PENITENTIARY: 'BA Penitentiary',
    cte.POLICE_STATION: 'BA Police Station',
    cte.POST_OFFICE: 'BA Post Office',
    cte.LIBRARY: 'BA Library',
    cte.EDUCATION: 'BA School/University',
    cte.PRIMARY_SCHOOL: 'BA School/University',
    cte.PRIMARY_SCHOOL_WITH_SHOWER: 'BA School/University',
    cte.SECONDARY_SCHOOL: 'BA School/University',
    cte.UNIVERSITY: 'BA School/University',
    cte.LABORATORY_AND_RESEARCH_CENTER: 'BA School/University',
    cte.STAND_ALONE_RETAIL: 'BA Retail',
    cte.HOSPITAL: 'BA Hospital',
    cte.OUT_PATIENT_HEALTH_CARE: 'BA Healthcare Clinic',
    cte.HEALTH_CARE: 'BA Healthcare Clinic',
    cte.RETIREMENT_HOME_OR_ORPHANAGE: 'BA Healthcare Clinic',
    cte.COMMERCIAL: 'BA Retail',
    cte.STRIP_MALL: 'BA Retail',
    cte.SUPERMARKET: 'BA Retail',
    cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD: 'BA Retail',
    cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD: 'BA Retail',
    cte.RESTAURANT: 'BA Dining: Bar Lounge/Leisure',
    cte.QUICK_SERVICE_RESTAURANT: 'BA Dining: Cafeteria/Fast Food',
    cte.FULL_SERVICE_RESTAURANT: 'BA Dining: Bar Lounge/Leisure',
    cte.HOTEL: 'BA Hotel',
    cte.HOTEL_MEDIUM_CLASS: 'BA Motel',
    cte.SMALL_HOTEL: 'BA Motel',
    cte.LARGE_HOTEL: 'BA Hotel',
    cte.DORMITORY: 'BA Dormitory',
    cte.EVENT_LOCATION: 'BA Convention Center',
    cte.CONVENTION_CENTER: 'BA Convention Center',
    cte.HALL: 'BA Town Hall',
    cte.GREEN_HOUSE: 'n/a',
    cte.INDUSTRY: 'BA Manufacturing Facility',
    cte.WORKSHOP: 'BA Workshop',
    cte.WAREHOUSE: 'BA Warehouse',
    cte.WAREHOUSE_REFRIGERATED: 'BA Warehouse',
    cte.SPORTS_LOCATION: 'BA Exercise Center',
    cte.SPORTS_ARENA: 'BA Sports Arena',
    cte.GYMNASIUM: 'BA Gymnasium',
    cte.MOTION_PICTURE_THEATRE: 'BA Motion Picture Theater',
    cte.MUSEUM: 'BA Museum',
    cte.PERFORMING_ARTS_THEATRE: 'BA Performing Arts Theater',
    cte.TRANSPORTATION: 'BA Transportation',
    cte.AUTOMOTIVE_FACILITY: 'BA Automotive Facility',
    cte.PARKING_GARAGE: 'BA Parking Garage',
    cte.RELIGIOUS: 'BA Religious Building',
    cte.NON_HEATED: 'n/a'
  }

  @property
  def dictionary(self) -> dict:
    return self._dictionary



"""
Location module
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez guillermo.gutierrezmorote@concordia.ca
Code contributors: Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""


class Location:
  """
  Location
  """
  def __init__(self, country, city):
    self._country = country
    self._city = city

  @property
  def city(self):
    """
    City name
    """
    return self._city

  @property
  def country(self):
    """
    Country code
    """
    return self._country



"""
Persistence (Postgresql) configuration
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Peter Yefi peteryefi@gmail.com
"""

import os
from dotenv import load_dotenv
from sqlalchemy.ext.declarative import declarative_base
from hub.hub_logger import logger

Models = declarative_base()

class Configuration:
  """
  Configuration class to hold common persistence configuration
  """

  def __init__(self, db_name: str, dotenv_path: str, app_env='TEST'):
    """
    :param db_name: database name
    :param app_env: application environment, test or production
    :param dotenv_path: the absolute path to dotenv file
    """
    try:
      # load environmental variables
      load_dotenv(dotenv_path=dotenv_path)
      self._db_name = db_name
      self._db_host = os.getenv(f'{app_env}_DB_HOST')
      self._db_user = os.getenv(f'{app_env}_DB_USER')
      self._db_pass = os.getenv(f'{app_env}_DB_PASSWORD')
      self._db_port = os.getenv(f'{app_env}_DB_PORT')
      self.hub_token = os.getenv('HUB_TOKEN')
    except KeyError as err:
      logger.error(f'Error with credentials: {err}')

  def conn_string(self):
    """
      Returns a connection string postgresql
      :return: connection string
      """
    if self._db_pass:
      return f'postgresql://{self._db_user}:{self._db_pass}@{self._db_host}:{self._db_port}/{self._db_name}'
    return f'postgresql://{self._db_user}@{self._db_host}:{self._db_port}/{self._db_name}'

  def get_db_user(self):
    return self._db_user



"""
Simplified Radiosity Algorithm
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guillermo.GutierrezMorote@concordia.ca
"""
import pandas as pd
import numpy as np
import calendar as cal
import hub.helpers.constants as cte


class SimplifiedRadiosityAlgorithm:
  """
  Import SRA results
  """
  def __init__(self, city, base_path):

    self._city = city
    self._base_path = base_path
    self._input_file_path = (self._base_path / f'{self._city.name}_sra_SW.out')
    self._month_hour = self._month_hour_data_frame
    self._results = self._read_results()
    self._radiation_list = []

  @property
  def _month_hour_data_frame(self):
    array = []
    for i in range(0, 12):
      days_of_month = cal.monthrange(2015, i+1)[1]
      total_hours = days_of_month * 24
      array = np.concatenate((array, np.full(total_hours, i + 1)))
    return pd.DataFrame(array, columns=[cte.MONTH])

  def _get_monthly_mean_values(self, values):
    out = None
    if values is not None:
      if cte.MONTH not in values.columns:
        values = pd.concat([self._month_hour, pd.DataFrame(values)], axis=1)
      out = values.groupby(cte.MONTH, as_index=False).mean()
      del out[cte.MONTH]
    return out

  def _get_yearly_mean_values(self, values):
    return values.mean()

  def _read_results(self):
    try:
      return pd.read_csv(self._input_file_path, sep='\s+', header=0)
    except Exception:
      raise Exception('No SRA output file found')

  @property
  def _radiation(self) -> []:
    if len(self._radiation_list) == 0:
      id_building = ''
      header_building = []
      for column in self._results.columns.values:
        if id_building != column.split(':')[1]:
          id_building = column.split(':')[1]
          if len(header_building) > 0:
            self._radiation_list.append(pd.concat([self._month_hour, self._results[header_building]],axis=1))
          header_building = [column]
        else:
          header_building.append(column)
      self._radiation_list.append(pd.concat([self._month_hour, self._results[header_building]], axis=1))
    return self._radiation_list

  def enrich(self):
    """
    saves in building surfaces the correspondent irradiance at different time-scales depending on the mode
    if building is None, it saves all buildings' surfaces in file, if building is specified, it saves only that
    specific building values
    :return: none
    """
    for radiation in self._radiation:
      city_object_name = radiation.columns.values.tolist()[1].split(':')[1]
      building = self._city.city_object(city_object_name)
      for column in radiation.columns.values:
        if column == cte.MONTH:
          continue
        header_id = column
        surface_id = header_id.split(':')[2]
        surface = building.surface_by_id(surface_id)
        new_value = pd.DataFrame(radiation[[header_id]].to_numpy(), columns=[cte.SRA])
        month_new_value = self._get_monthly_mean_values(new_value)
        if cte.MONTH not in surface.global_irradiance:
          surface.global_irradiance[cte.MONTH] = month_new_value
        else:
          pd.concat([surface.global_irradiance[cte.MONTH], month_new_value], axis=1)
        if cte.HOUR not in surface.global_irradiance:
          surface.global_irradiance[cte.HOUR] = new_value
        else:
          pd.concat([surface.global_irradiance[cte.HOUR], new_value], axis=1)
        if cte.YEAR not in surface.global_irradiance:
          surface.global_irradiance[cte.YEAR] = self._get_yearly_mean_values(new_value)
    self._city.level_of_detail.surface_radiation = 2



"""
Yearly from daily schedules module
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""
import calendar as cal
import hub.helpers.constants as cte
from hub.city_model_structure.attributes.schedule import Schedule


class YearlyFromDailySchedules:
  """
  YearlyFromDailySchedules class
  """
  def __init__(self, daily_schedules, year):
    self._daily_schedules = daily_schedules
    self._year = year

  @property
  def yearly_schedule(self) -> Schedule:
    """
    Creates a yearly schedule out of a set of daily schedules
    :return: Schedule
    """
    yearly_schedule = Schedule()
    weekly_schedules = [0, 0, 0, 0, 0, 0, 0]
    day_types = dict({cte.MONDAY: 0, cte.TUESDAY: 1, cte.WEDNESDAY: 2, cte.THURSDAY: 3,
                      cte.FRIDAY: 4, cte.SATURDAY: 5, cte.SUNDAY: 6})
    for daily_schedule in self._daily_schedules:
      for day_type in daily_schedule.day_types:
        weekly_schedules[day_types[day_type]] = daily_schedule.values

    values = []
    for month in range(1, 13):
      _, number_days = cal.monthrange(self._year, month)
      for day in range(1, number_days+1):
        week_day = cal.weekday(self._year, month, day)
        values.extend(weekly_schedules[week_day])
    yearly_schedule.type = self._daily_schedules[0].type
    yearly_schedule.data_type = self._daily_schedules[0].data_type
    yearly_schedule.time_range = cte.YEAR
    yearly_schedule.time_step = cte.HOUR
    yearly_schedule.values = values

    return yearly_schedule



"""
Storey module
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""

from __future__ import annotations
from typing import List
from hub.city_model_structure.building_demand.surface import Surface
from hub.city_model_structure.building_demand.thermal_boundary import ThermalBoundary
from hub.city_model_structure.building_demand.thermal_zone import ThermalZone


class Storey:
  """
  Storey class
  """
  def __init__(self, name, storey_surfaces, neighbours, volume, internal_zone, floor_area):
    self._name = name
    self._storey_surfaces = storey_surfaces
    self._thermal_boundaries = None
    self._virtual_surfaces = None
    self._thermal_zone = None
    self._neighbours = neighbours
    self._volume = volume
    self._internal_zone = internal_zone
    self._floor_area = floor_area

  @property
  def name(self):
    """
    Get storey's name
    :return: str
    """
    return self._name

  @property
  def surfaces(self) -> List[Surface]:
    """
    Get external surfaces enclosing the storey
    :return: [Surface]
    """
    return self._storey_surfaces

  @property
  def neighbours(self):
    """
    Get the neighbour storeys' names
    :return: [str]
    """
    return self._neighbours

  @property
  def thermal_boundaries(self) -> List[ThermalBoundary]:
    """
    Get the thermal boundaries bounding the thermal zone
    :return: [ThermalBoundary]
    """
    if self._thermal_boundaries is None:
      self._thermal_boundaries = []
      for surface in self.surfaces:
        if surface.holes_polygons is None:
          windows_areas = None
        else:
          windows_areas = []
          for hole in surface.holes_polygons:
            windows_areas.append(hole.area)
        new_thermal_boundary = ThermalBoundary(surface, surface.solid_polygon.area, windows_areas)
        surface.associated_thermal_boundaries.append(new_thermal_boundary)
        self._thermal_boundaries.append(new_thermal_boundary)
    return self._thermal_boundaries

  @property
  def virtual_surfaces(self) -> List[Surface]:
    """
    Get the internal surfaces enclosing the thermal zone
    :return: [Surface]
    """
    if self._virtual_surfaces is None:
      self._virtual_surfaces = []
      for thermal_boundary in self.thermal_boundaries:
        self._virtual_surfaces.append(thermal_boundary.internal_surface)
    return self._virtual_surfaces

  @property
  def thermal_zone(self) -> ThermalZone:
    """
    Get the thermal zone inside the storey
    :return: ThermalZone
    """
    if self._thermal_zone is None:
      self._thermal_zone = ThermalZone(self.thermal_boundaries, self._internal_zone, self.volume, self.floor_area)
    return self._thermal_zone

  @property
  def volume(self):
    """
    Get storey's volume in cubic meters
    :return: float
    """
    return self._volume

  @property
  def floor_area(self):
    """
    Get storey's floor area in square meters
    :return: float
    """
    return self._floor_area



"""
Dictionaries module saves all transformations of functions and usages to access the catalogs
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""

from hub.helpers.data.hft_function_to_hub_function import HftFunctionToHubFunction
from hub.helpers.data.montreal_function_to_hub_function import MontrealFunctionToHubFunction
from hub.helpers.data.alkis_function_to_hub_function import AlkisFunctionToHubFunction
from hub.helpers.data.pluto_function_to_hub_function import PlutoFunctionToHubFunction
from hub.helpers.data.hub_function_to_nrel_construction_function import HubFunctionToNrelConstructionFunction
from hub.helpers.data.hub_function_to_nrcan_construction_function import HubFunctionToNrcanConstructionFunction
from hub.helpers.data.hub_usage_to_comnet_usage import HubUsageToComnetUsage
from hub.helpers.data.hub_usage_to_hft_usage import HubUsageToHftUsage
from hub.helpers.data.hub_usage_to_nrcan_usage import HubUsageToNrcanUsage

class Dictionaries:
  """
  Dictionaries class
  """

  @property
  def hub_usage_to_hft_usage(self) -> dict:
    """
    Hub usage to HfT usage, transformation dictionary
    :return: dict
    """
    return HubUsageToHftUsage().dictionary

  @property
  def hub_usage_to_comnet_usage(self) -> dict:
    """
    Hub usage to Comnet usage, transformation dictionary
    :return: dict
    """
    return HubUsageToComnetUsage().dictionary

  @property
  def hub_usage_to_nrcan_usage(self) -> dict:
    """
    Get hub usage to NRCAN usage, transformation dictionary
    :return: dict
    """
    return HubUsageToNrcanUsage().dictionary

  @property
  def hub_function_to_nrcan_construction_function(self) -> dict:
    """
    Get hub function to NRCAN construction function, transformation dictionary
    :return: dict
    """
    return HubFunctionToNrcanConstructionFunction().dictionary

  @property
  def hub_function_to_nrel_construction_function(self) -> dict:
    """
    Get hub function to NREL construction function, transformation dictionary
    :return: dict
    """
    return HubFunctionToNrelConstructionFunction().dictionary

  @property
  def pluto_function_to_hub_function(self) -> dict:
    """
    Get Pluto function to hub function, transformation dictionary
    :return: dict
    """
    return PlutoFunctionToHubFunction().dictionary

  @property
  def hft_function_to_hub_function(self) -> dict:
    """
    Get Hft function to hub function, transformation dictionary
    :return: dict
    """
    return HftFunctionToHubFunction().dictionary

  @property
  def montreal_function_to_hub_function(self) -> dict:
    """
    Get Montreal function to hub function, transformation dictionary
    """
    return MontrealFunctionToHubFunction().dictionary

  @property
  def alkis_function_to_hub_function(self) -> dict:
    """
    Get Alkis function to hub function, transformation dictionary
    """
    return AlkisFunctionToHubFunction().dictionary




"""
Dictionaries module for Hft function to hub function
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez Guillermo.GutierrezMorote@concordia.ca
"""

import hub.helpers.constants as cte


class HftFunctionToHubFunction:

  def __init__(self):
    self._dictionary = {
    'residential': cte.RESIDENTIAL,
    'single family house': cte.SINGLE_FAMILY_HOUSE,
    'multifamily house': cte.MULTI_FAMILY_HOUSE,
    'hotel': cte.HOTEL,
    'hospital': cte.HOSPITAL,
    'outpatient': cte.OUT_PATIENT_HEALTH_CARE,
    'commercial': cte.SUPERMARKET,
    'strip mall': cte.STRIP_MALL,
    'warehouse': cte.WAREHOUSE,
    'primary school': cte.PRIMARY_SCHOOL,
    'secondary school': cte.EDUCATION,
    'office': cte.MEDIUM_OFFICE,
    'large office': cte.LARGE_OFFICE
  }

  @property
  def dictionary(self)  -> dict:
    return self._dictionary



"""
ConstructionFactory (before PhysicsFactory) retrieve the specific construction module for the given region
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez guillermo.gutierrezmorote@concordia.ca
Code contributors: Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""

from pathlib import Path
from hub.hub_logger import logger
from hub.helpers.utils import validate_import_export_type
from hub.imports.construction.nrel_physics_parameters import NrelPhysicsParameters
from hub.imports.construction.nrcan_physics_parameters import NrcanPhysicsParameters


class ConstructionFactory:
  """
  ConstructionFactory class
  """
  def __init__(self, handler, city, base_path=None):
    if base_path is None:
      base_path = Path(Path(__file__).parent.parent / 'data/construction')
    self._handler = '_' + handler.lower().replace(' ', '_')
    class_funcs = validate_import_export_type(ConstructionFactory)
    if self._handler not in class_funcs:
      err_msg = f"Wrong import type [{self._handler}]. Valid functions include {class_funcs}"
      logger.error(err_msg)
      raise Exception(err_msg)
    self._city = city
    self._base_path = base_path

  def _nrel(self):
    """
    Enrich the city by using NREL information
    """
    NrelPhysicsParameters(self._city, self._base_path).enrich_buildings()
    self._city.level_of_detail.construction = 2

  def _nrcan(self):
    """
    Enrich the city by using NRCAN information
    """
    NrcanPhysicsParameters(self._city, self._base_path).enrich_buildings()
    self._city.level_of_detail.construction = 2

  def enrich(self):
    """
    Enrich the city given to the class using the class given handler
    :return: None
    """
    getattr(self, self._handler, lambda: None)()

  def enrich_debug(self):
    """
    Enrich the city given to the class using the class given handler
    :return: None
    """
    NrelPhysicsParameters(self._city, self._base_path).enrich_buildings()



"""
CityGmlLod1 module parses citygml_classes files with level of detail 1 and import the geometry into the city model
structure
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez guillermo.gutierrezmorote@concordia.ca
"""

from hub.imports.geometry.citygml_classes.citygml_base import CityGmlBase
from hub.imports.geometry.helpers.geometry_helper import GeometryHelper
from hub.city_model_structure.building_demand.surface import Surface
from hub.city_model_structure.attributes.polygon import Polygon


class CityGmlLod2(CityGmlBase):
  """
  CityGmlLod1 class to parse level of detail 1 city gml files
  """
  def __init__(self, city_object_member):
    super().__init__()
    self._city_object_member = city_object_member
    self._surfaces = self._identify(self._city_object_member)

  @classmethod
  def _identify(cls, city_object_member):
    if 'lod2Solid' in city_object_member:
      return cls._solid(city_object_member)
    if 'lod2MultiSurface' in city_object_member:
      return cls._multi_surface(city_object_member)
    if 'lod2MultiCurve' in city_object_member:
      return cls._multi_curve(city_object_member)
    raise NotImplementedError(city_object_member)

  @staticmethod
  def _surface_encoding(surfaces):
    if 'lod2MultiSurface' in surfaces:
      return 'lod2MultiSurface', 'MultiSurface'
    raise NotImplementedError('unknown surface type')

  @classmethod
  def _solid(cls, city_object_member):
    surfaces = []
    for bounded in city_object_member["boundedBy"]:
      try:
        surface_type = next(iter(bounded))
      except TypeError:
        continue
      try:
        surface_encoding, surface_subtype = cls._surface_encoding(bounded[surface_type])
      except NotImplementedError:
        continue
      for member in bounded[surface_type][surface_encoding][surface_subtype]['surfaceMember']:
        if 'CompositeSurface' in member:
          for composite_members in member['CompositeSurface']['surfaceMember']:
            for composite_member in composite_members['CompositeSurface']['surfaceMember']:
              surfaces.append(cls._add_member_surface(composite_member, surface_type))
        else:
          surfaces.append(cls._add_member_surface(member, surface_type))
    return surfaces

  @classmethod
  def _add_member_surface(cls, member, surface_type):
    pos_name = 'posList'
    if pos_name not in member['Polygon']['exterior']['LinearRing']:
      pos_name = 'pos'
    if '@srsDimension' in member['Polygon']['exterior']['LinearRing'][pos_name]:
      gml_points = member['Polygon']['exterior']['LinearRing']['posList']["#text"]
    else:
      gml_points = member['Polygon']['exterior']['LinearRing'][pos_name]
    if pos_name == 'pos':
      gml_points_string = ''
      for gml_point in gml_points:
        gml_points_string = f'{gml_points_string} {gml_point}'
      gml_points = gml_points_string.lstrip(' ')
    solid_points = GeometryHelper.points_from_string(GeometryHelper.remove_last_point_from_string(gml_points))
    polygon = Polygon(solid_points)
    return Surface(polygon, polygon, surface_type=GeometryHelper.gml_surface_to_libs(surface_type))

  @classmethod
  def _multi_curve(cls, city_object_member):
    raise NotImplementedError('multi curve')

  @classmethod
  def _multi_surface(cls, city_object_member):
    return cls._solid(city_object_member)



"""
Building module
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez guillermo.gutierrezmorote@concordia.ca
Code contributors: Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""

import sys
from typing import List, Union
import numpy as np
from hub.hub_logger import logger
import hub.helpers.constants as cte
from hub.city_model_structure.building_demand.surface import Surface
from hub.city_model_structure.city_object import CityObject
from hub.city_model_structure.building_demand.household import Household
from hub.city_model_structure.building_demand.internal_zone import InternalZone
from hub.city_model_structure.attributes.polyhedron import Polyhedron


class Building(CityObject):
  """
  Building(CityObject) class
  """
  def __init__(self, name, surfaces, year_of_construction, function, terrains=None):
    super().__init__(name, surfaces)
    self._households = None
    self._basement_heated = None
    self._attic_heated = None
    self._terrains = terrains
    self._year_of_construction = year_of_construction
    self._function = function
    self._average_storey_height = None
    self._storeys_above_ground = None
    self._floor_area = None
    self._roof_type = None
    self._internal_zones = None
    self._shell = None
    self._alias = None
    self._type = 'building'
    self._heating = dict()
    self._cooling = dict()
    self._lighting_electrical_demand = dict()
    self._appliances_electrical_demand = dict()
    self._domestic_hot_water_heat_demand = dict()
    self._eave_height = None
    self._grounds = []
    self._roofs = []
    self._walls = []
    self._internal_walls = []
    self._ground_walls = []
    self._attic_floors = []
    self._interior_slabs = []
    for surface_id, surface in enumerate(self.surfaces):
      self._min_x = min(self._min_x, surface.lower_corner[0])
      self._min_y = min(self._min_y, surface.lower_corner[1])
      self._min_z = min(self._min_z, surface.lower_corner[2])
      surface.id = surface_id
      if surface.type == cte.GROUND:
        self._grounds.append(surface)
      elif surface.type == cte.WALL:
        self._walls.append(surface)
      elif surface.type == cte.ROOF:
        self._roofs.append(surface)
      elif surface.type == cte.INTERIOR_WALL:
        self._internal_walls.append(surface)
      elif surface.type == cte.GROUND_WALL:
        self._ground_walls.append(surface)
      elif surface.type == cte.ATTIC_FLOOR:
        self._attic_floors.append(surface)
      elif surface.type == cte.INTERIOR_SLAB:
        self._interior_slabs.append(surface)
      else:
        logger.error(f'Building {self.name} [alias {self.alias}]  has an unexpected surface type {surface.type}.\n')
        sys.stderr.write(f'Building {self.name} [alias {self.alias}]  has an unexpected surface type {surface.type}.\n')

  @property
  def shell(self) -> Polyhedron:
    """
    Get building's external polyhedron
    :return: [Polyhedron]
    """
    polygons = []
    for surface in self.surfaces:
      if surface.type is not cte.INTERIOR_WALL:
        polygons.append(surface.solid_polygon)
        if surface.holes_polygons is not None:
          for hole in surface.holes_polygons:
            polygons.append(hole)
    if self._shell is None:
      self._shell = Polyhedron(polygons)
    return self._shell

  @property
  def internal_zones(self) -> List[InternalZone]:
    """
    Get building internal zones
    For Lod up to 3, there is only one internal zone which corresponds to the building shell.
    In LoD 4 there can be more than one. In this case the definition of surfaces and floor area must be redefined.
    :return: [InternalZone]
    """
    if self._internal_zones is None:
      self._internal_zones = [InternalZone(self.surfaces, self.floor_area)]
    return self._internal_zones

  @property
  def grounds(self) -> List[Surface]:
    """
    Get building ground surfaces
    :return: [Surface]
    """
    return self._grounds

  @property
  def roofs(self) -> List[Surface]:
    """
    Get building roof surfaces
    :return: [Surface]
    """
    return self._roofs

  @property
  def walls(self) -> List[Surface]:
    """
    Get building wall surfaces
    :return: [Surface]
    """
    return self._walls

  @property
  def internal_walls(self) -> List[Surface]:
    """
    Get building internal wall surfaces
    :return: [Surface]
    """
    return self._internal_walls

  @property
  def terrains(self) -> Union[None, List[Surface]]:
    """
    Get city object terrain surfaces
    :return: [Surface]
    """
    return self._terrains

  @property
  def attic_heated(self) -> Union[None, int]:
    """
    Get if the city object attic is heated
    0: no attic in the building
    1: attic exists but is not heated
    2: attic exists and is heated
    :return: None or int
    """
    return self._attic_heated

  @attic_heated.setter
  def attic_heated(self, value):
    """
    Set if the city object attic is heated
    0: no attic in the building
    1: attic exists but is not heated
    2: attic exists and is heated
    :param value: int
    """
    if value is not None:
      self._attic_heated = int(value)

  @property
  def basement_heated(self) -> Union[None, int]:
    """
    Get if the city object basement is heated
    0: no basement in the building
    1: basement exists but is not heated
    2: basement exists and is heated
    :return: None or int
    """
    return self._basement_heated

  @basement_heated.setter
  def basement_heated(self, value):
    """
    Set if the city object basement is heated
    0: no basement in the building
    1: basement exists but is not heated
    2: basement exists and is heated
    :param value: int
    """
    if value is not None:
      self._basement_heated = int(value)

  @property
  def heated_volume(self):
    """
    Raises not implemented error
    """
    # todo: this need to be calculated based on the basement and attic heated values
    raise NotImplementedError

  @property
  def year_of_construction(self):
    """
    Get building year of construction
    :return: int
    """
    return self._year_of_construction

  @year_of_construction.setter
  def year_of_construction(self, value):
    """
    Set building year of construction
    :param value: int
    """
    if value is not None:
      self._year_of_construction = int(value)

  @property
  def function(self) -> Union[None, str]:
    """
    Get building function
    :return: None or str
    """
    return self._function

  @function.setter
  def function(self, value):
    """
    Set building function
    :param value: str
    """
    if value is not None:
      self._function = str(value)

  @property
  def average_storey_height(self) -> Union[None, float]:
    """
    Get building average storey height in meters
    :return: None or float
    """
    return self._average_storey_height

  @average_storey_height.setter
  def average_storey_height(self, value):
    """
    Set building average storey height in meters
    :param value: float
    """
    if value is not None:
      self._average_storey_height = float(value)

  @property
  def storeys_above_ground(self) -> Union[None, int]:
    """
    Get building storeys number above ground
    :return: None or int
    """
    return self._storeys_above_ground

  @storeys_above_ground.setter
  def storeys_above_ground(self, value):
    """
    Set building storeys number above ground
    :param value: int
    """
    if value is not None:
      self._storeys_above_ground = int(value)

  @property
  def heating(self) -> dict:
    """
    Get heating demand in Wh
    :return: dict{DataFrame(float)}
    """
    return self._heating

  @heating.setter
  def heating(self, value):
    """
    Set heating demand in Wh
    :param value: dict{DataFrame(float)}
    """
    self._heating = value

  @property
  def cooling(self) -> dict:
    """
    Get cooling demand in Wh
    :return: dict{DataFrame(float)}
    """
    return self._cooling

  @cooling.setter
  def cooling(self, value):
    """
    Set cooling demand in Wh
    :param value: dict{DataFrame(float)}
    """
    self._cooling = value

  @property
  def lighting_electrical_demand(self) -> dict:
    """
    Get lighting electrical demand in Wh
    :return: dict{DataFrame(float)}
    """
    return self._lighting_electrical_demand

  @lighting_electrical_demand.setter
  def lighting_electrical_demand(self, value):
    """
    Set lighting electrical demand in Wh
    :param value: dict{DataFrame(float)}
    """
    self._lighting_electrical_demand = value

  @property
  def appliances_electrical_demand(self) -> dict:
    """
    Get appliances electrical demand in Wh
    :return: dict{DataFrame(float)}
    """
    return self._appliances_electrical_demand

  @appliances_electrical_demand.setter
  def appliances_electrical_demand(self, value):
    """
    Set appliances electrical demand in Wh
    :param value: dict{DataFrame(float)}
    """
    self._appliances_electrical_demand = value

  @property
  def domestic_hot_water_heat_demand(self) -> dict:
    """
    Get domestic hot water heat demand in Wh
    :return: dict{DataFrame(float)}
    """
    return self._domestic_hot_water_heat_demand

  @domestic_hot_water_heat_demand.setter
  def domestic_hot_water_heat_demand(self, value):
    """
    Set domestic hot water heat demand in Wh
    :param value: dict{DataFrame(float)}
    """
    self._domestic_hot_water_heat_demand = value

  @property
  def eave_height(self):
    """
    Get building eave height in meters
    :return: float
    """
    if self._eave_height is None:
      self._eave_height = 0
      for wall in self.walls:
        self._eave_height = max(self._eave_height, wall.upper_corner[2])
    return self._eave_height

  @property
  def roof_type(self):
    """
    Get roof type for the building flat or pitch
    :return: str
    """
    if self._roof_type is None:
      self._roof_type = 'flat'
      for roof in self.roofs:
        grads = np.rad2deg(roof.inclination)
        if 355 > grads > 5:
          self._roof_type = 'pitch'
          break
    return self._roof_type

  @roof_type.setter
  def roof_type(self, value):
    """
    Set roof type for the building flat or pitch
    :return: str
    """
    self._roof_type = value

  @property
  def floor_area(self):
    """
    Get building floor area in square meters
    :return: float
    """
    if self._floor_area is None:
      self._floor_area = 0
      for surface in self.surfaces:
        if surface.type == 'Ground':
          self._floor_area += surface.perimeter_polygon.area
    return self._floor_area

  @property
  def households(self) -> List[Household]:
    """
    Get the list of households inside the building
    :return: List[Household]
    """
    return self._households

  @property
  def is_conditioned(self):
    """
    Get building heated flag
    :return: Boolean
    """
    if self.internal_zones is None:
      return False
    for internal_zone in self.internal_zones:
      if internal_zone.usages is not None:
        for usage in internal_zone.usages:
          if usage.thermal_control is not None:
            return True
    return False

  @property
  def alias(self):
    """
    Get the alias name for the building
    :return: str
    """
    return self._alias

  @alias.setter
  def alias(self, value):
    """
    Set the alias name for the building
    """
    self._alias = value

  @property
  def usages_percentage(self):
    """
    Get the usages and percentages for the building
    """
    _usage = ''
    for internal_zone in self.internal_zones:
      for usage in internal_zone.usages:
        _usage = f'{_usage}{usage.name}_{usage.percentage} '
    return _usage.rstrip()



"""
Usage catalog appliances
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez guillermo.gutierrezmorote@concordia.ca
"""

from typing import Union, List

from hub.catalog_factories.data_models.usages.schedule import Schedule


class Appliances:
  """
  Appliances class
  """
  def __init__(self, density, convective_fraction, radiative_fraction, latent_fraction, schedules):
    self._density = density
    self._convective_fraction = convective_fraction
    self._radiative_fraction = radiative_fraction
    self._latent_fraction = latent_fraction
    self._schedules = schedules

  @property
  def density(self) -> Union[None, float]:
    """
    Get appliances density in Watts per m2
    :return: None or float
    """
    return self._density

  @property
  def convective_fraction(self) -> Union[None, float]:
    """
    Get convective fraction
    :return: None or float
    """
    return self._convective_fraction

  @property
  def radiative_fraction(self) -> Union[None, float]:
    """
    Get radiant fraction
    :return: None or float
    """
    return self._radiative_fraction

  @property
  def latent_fraction(self) -> Union[None, float]:
    """
    Get latent fraction
    :return: None or float
    """
    return self._latent_fraction

  @property
  def schedules(self) -> Union[None, List[Schedule]]:
    """
    Get schedules
    dataType = fraction
    :return: None or [Schedule]
    """
    return self._schedules



"""
InternalZone module. It saves the original geometrical information from interiors together with some attributes of those
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""

import uuid
from typing import Union, List
from hub.city_model_structure.building_demand.usage import Usage
from hub.city_model_structure.building_demand.thermal_zone import ThermalZone
from hub.city_model_structure.attributes.polyhedron import Polyhedron
from hub.city_model_structure.energy_systems.hvac_system import HvacSystem


class InternalZone:
  """
  InternalZone class
  """
  def __init__(self, surfaces, area):
    self._surfaces = surfaces
    self._id = None
    self._geometry = None
    self._volume = None
    self._area = area
    self._thermal_zones = None
    self._usages = None
    self._hvac_system = None

  @property
  def id(self):
    """
    Get internal zone id, a universally unique identifier randomly generated
    :return: str
    """
    if self._id is None:
      self._id = uuid.uuid4()
    return self._id

  @property
  def geometry(self) -> Polyhedron:
    """
    Get internal zone geometry
    :return: Polyhedron
    """
    if self._geometry is None:
      polygons = []
      for surface in self.surfaces:
        polygons.append(surface.perimeter_polygon)
      self._geometry = Polyhedron(polygons)
    return self._geometry

  @property
  def surfaces(self):
    """
    Get internal zone surfaces
    :return: [Surface]
    """
    return self._surfaces

  @property
  def volume(self):
    """
    Get internal zone volume in cubic meters
    :return: float
    """
    return self.geometry.volume

  @property
  def area(self):
    """
    Get internal zone area in square meters
    :return: float
    """
    return self._area

  @property
  def usages(self) -> [Usage]:
    """
    Get internal zone usage zones
    :return: [UsageZone]
    """
    return self._usages

  @usages.setter
  def usages(self, value):
    """
    Set internal zone usage zones
    :param value: [UsageZone]
    """
    self._usages = value

  @property
  def hvac_system(self) -> Union[None, HvacSystem]:
    """
    Get HVAC system installed for this thermal zone
    :return: None or HvacSystem
    """
    return self._hvac_system

  @hvac_system.setter
  def hvac_system(self, value):
    """
    Set HVAC system installed for this thermal zone
    :param value: HvacSystem
    """
    self._hvac_system = value

  @property
  def thermal_zones(self) -> Union[None, List[ThermalZone]]:
    """
    Get building thermal zones
    :return: [ThermalZone]
    """
    return self._thermal_zones

  @thermal_zones.setter
  def thermal_zones(self, value):
    """
    Set city object thermal zones
    :param value: [ThermalZone]
    """
    self._thermal_zones = value



"""
Comnet usage catalog
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez guillermo.gutierrezmorote@concordia.ca
"""
from typing import Dict

import pandas as pd

import hub.helpers.constants as cte
from hub.catalog_factories.catalog import Catalog
from hub.catalog_factories.data_models.usages.appliances import Appliances
from hub.catalog_factories.data_models.usages.content import Content
from hub.catalog_factories.data_models.usages.lighting import Lighting
from hub.catalog_factories.data_models.usages.ocupancy import Occupancy
from hub.catalog_factories.data_models.usages.schedule import Schedule
from hub.catalog_factories.data_models.usages.thermal_control import ThermalControl
from hub.catalog_factories.data_models.usages.usage import Usage
from hub.catalog_factories.usage.usage_helper import UsageHelper
from hub.helpers.configuration_helper import ConfigurationHelper as ch


class ComnetCatalog(Catalog):
  def __init__(self, path):
    self._comnet_archetypes_path = str(path / 'comnet_archetypes.xlsx')
    self._comnet_schedules_path = str(path / 'comnet_schedules_archetypes.xlsx')
    self._archetypes = self._read_archetype_file()
    self._schedules = self._read_schedules_file()

    # todo: comment with @Guille, this hypotheses should go in the import factory?
    sensible_convective = ch().comnet_occupancy_sensible_convective
    sensible_radiative = ch().comnet_occupancy_sensible_radiant
    lighting_convective = ch().comnet_lighting_convective
    lighting_radiative = ch().comnet_lighting_radiant
    lighting_latent = ch().comnet_lighting_latent
    appliances_convective = ch().comnet_plugs_convective
    appliances_radiative = ch().comnet_plugs_radiant
    appliances_latent = ch().comnet_plugs_latent

    usages = []
    for schedule_key in self._archetypes['schedules_key']:
      comnet_usage = schedule_key
      schedule_name = self._archetypes['schedules_key'][schedule_key]
      hours_day = None
      days_year = None
      occupancy_archetype = self._archetypes['occupancy'][comnet_usage]
      lighting_archetype = self._archetypes['lighting'][comnet_usage]
      appliances_archetype = self._archetypes['plug loads'][comnet_usage]
      mechanical_air_change = None  # comnet provides ventilation rate only
      ventilation_rate = self._archetypes['ventilation rate'][comnet_usage]
      # convert cfm/ft2 to m3/m2.s
      ventilation_rate = ventilation_rate / (cte.METERS_TO_FEET * cte.MINUTES_TO_SECONDS)

      # get occupancy
      occupancy_density = occupancy_archetype[0] / pow(cte.METERS_TO_FEET, 2)
      sensible_heat_gain = occupancy_archetype[1] * cte.BTU_H_TO_WATTS
      latent_heat_gain = occupancy_archetype[1] * cte.BTU_H_TO_WATTS
      if occupancy_density != 0:
        occupancy_density = 1 / occupancy_density
      sensible_convective_internal_gain = occupancy_density * sensible_heat_gain * sensible_convective
      sensible_radiative_internal_gain = occupancy_density * sensible_heat_gain * sensible_radiative
      latent_internal_gain = occupancy_density * latent_heat_gain
      occupancy = Occupancy(occupancy_density,
                            sensible_convective_internal_gain,
                            sensible_radiative_internal_gain,
                            latent_internal_gain,
                            self._schedules[schedule_name]['Occupancy'])

      # get lighting
      density = lighting_archetype[4] * pow(cte.METERS_TO_FEET, 2)
      lighting = Lighting(density,
                          lighting_convective,
                          lighting_radiative,
                          lighting_latent,
                          self._schedules[schedule_name]['Lights'])

      # get appliances
      density = appliances_archetype[0]
      if density == 'n.a.':
        density = 0
      # convert W/ft2 to W/m2
      density = float(density) * pow(cte.METERS_TO_FEET, 2)
      appliances = Appliances(density,
                              appliances_convective,
                              appliances_radiative,
                              appliances_latent,
                              self._schedules[schedule_name]['Receptacle'])

      # get thermal control
      thermal_control = ThermalControl(None,
                                       None,
                                       None,
                                       self._schedules[schedule_name]['HVAC Avail'],
                                       self._schedules[schedule_name]['HtgSetPt'],
                                       self._schedules[schedule_name]['ClgSetPt']
                                       )

      usages.append(Usage(comnet_usage,
                          hours_day,
                          days_year,
                          mechanical_air_change,
                          ventilation_rate,
                          occupancy,
                          lighting,
                          appliances,
                          thermal_control))

    self._content = Content(usages)

  def _read_schedules_file(self) -> Dict:
    dictionary = {}
    comnet_usages = UsageHelper().comnet_schedules_key_to_comnet_schedules
    comnet_days = UsageHelper().comnet_days
    comnet_data_types = UsageHelper().comnet_data_type_to_hub_data_type
    for usage_name in comnet_usages:
      if usage_name == 'C-13 Data Center':
        continue
      _extracted_data = pd.read_excel(self._comnet_schedules_path, sheet_name=comnet_usages[usage_name],
                                      skiprows=[0, 1, 2, 3], nrows=39, usecols="A:AA")
      _schedules = {}
      for row in range(0, 39, 3):
        _schedule_values = {}
        schedule_name = _extracted_data.loc[row:row, 'Description'].item()
        schedule_data_type = comnet_data_types[_extracted_data.loc[row:row, 'Type'].item()]
        for day in comnet_days:
          # Monday to Friday
          start = row
          end = row + 1
          if day == cte.SATURDAY:
            start = start + 1
            end = end + 1
          elif day == cte.SUNDAY or day == cte.HOLIDAY:
            start = start + 2
            end = end + 2
          _schedule_values[day] = _extracted_data.iloc[start:end, 3:27].to_numpy().tolist()[0]
        _schedule = []
        for day in _schedule_values:
          if schedule_name == 'ClgSetPt' or schedule_name == 'HtgSetPt':
            # to celsius
            if 'n.a.' in _schedule_values[day]:
              _schedule_values[day] = None
            else:
              _schedule_values[day] = [(float(value)-32)*5/9 for value in _schedule_values[day]]
          _schedule.append(Schedule(schedule_name, _schedule_values[day], schedule_data_type, cte.HOUR, cte.DAY, [day]))
        _schedules[schedule_name] = _schedule
      dictionary[usage_name] = _schedules
    return dictionary

  def _read_archetype_file(self) -> Dict:
    """
    reads xlsx files containing usage information into a dictionary
    :return : Dict
    """
    number_usage_types = 33
    xl_file = pd.ExcelFile(self._comnet_archetypes_path)
    file_data = pd.read_excel(xl_file, sheet_name="Modeling Data", skiprows=[0, 1, 2, 24],
                              nrows=number_usage_types, usecols="A:AB")

    lighting_data = {}
    plug_loads_data = {}
    occupancy_data = {}
    ventilation_rate = {}
    water_heating = {}
    process_data = {}
    schedules_key = {}
    for j in range(0, number_usage_types-1):
      usage_parameters = file_data.iloc[j]
      usage_type = usage_parameters[0]
      lighting_data[usage_type] = usage_parameters[1:6].values.tolist()
      plug_loads_data[usage_type] = usage_parameters[8:13].values.tolist()
      occupancy_data[usage_type] = usage_parameters[17:20].values.tolist()
      ventilation_rate[usage_type] = usage_parameters[20:21].item()
      water_heating[usage_type] = usage_parameters[23:24].item()
      process_data[usage_type] = usage_parameters[24:26].values.tolist()
      schedules_key[usage_type] = usage_parameters[27:28].item()

    return {'lighting': lighting_data,
            'plug loads': plug_loads_data,
            'occupancy': occupancy_data,
            'ventilation rate': ventilation_rate,
            'water heating': water_heating,
            'process': process_data,
            'schedules_key': schedules_key
            }

  def names(self, category=None):
    """
    Get the catalog elements names
    :parm: for usage catalog category filter does nothing as there is only one category (usages)
    """
    _names = {'usages': []}
    for usage in self._content.usages:
      _names['usages'].append(usage.name)
    return _names

  def entries(self, category=None):
    """
    Get the catalog elements
    :parm: for usage catalog category filter does nothing as there is only one category (usages)
    """
    return self._content

  def get_entry(self, name):
    """
    Get one catalog element by names
    :parm: entry name
    """
    for usage in self._content.usages:
      if usage.name.lower() == name.lower():
        return usage
    raise IndexError(f"{name} doesn't exists in the catalog")



"""
NRCAN construction catalog
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""

import json
import urllib.request
import xmltodict

from hub.catalog_factories.catalog import Catalog
from hub.catalog_factories.data_models.usages.content import Content
from hub.catalog_factories.construction.construction_helper import ConstructionHelper
from hub.catalog_factories.data_models.construction.construction import Construction
from hub.catalog_factories.data_models.construction.archetype import Archetype


class NrcanCatalog(Catalog):
  def __init__(self, path):
    path = str(path / 'nrcan.xml')
    self._content = None
    self._g_value_per_hdd = []
    self._thermal_transmittance_per_hdd_and_surface = {}
    self._window_ratios = {}
    with open(path) as xml:
      self._metadata = xmltodict.parse(xml.read())
    self._base_url_archetypes = self._metadata['nrcan']['@base_url_archetypes']
    self._base_url_construction = self._metadata['nrcan']['@base_url_construction']
    self._load_window_ratios()
    self._load_construction_values()
    self._content = Content(self._load_archetypes())

  def _load_window_ratios(self):
    for standard in self._metadata['nrcan']['standards_per_function']['standard']:
      url = f'{self._base_url_archetypes}{standard["file_location"]}'
    # todo: read from file
    self._window_ratios = {'Mean': 0.2, 'North': 0.2, 'East': 0.2, 'South': 0.2, 'West': 0.2}

  def _load_construction_values(self):
    for standard in self._metadata['nrcan']['standards_per_period']['standard']:
      g_value_url = f'{self._base_url_construction}{standard["g_value_location"]}'
      punc = '()<?:'
      with urllib.request.urlopen(g_value_url) as json_file:
        text = json.load(json_file)['tables']['SHGC']['table'][0]['formula']
        values = ''.join([o for o in list(text) if o not in punc]).split()
        for index in range(int((len(values) - 1)/3)):
          self._g_value_per_hdd.append([values[3*index+1], values[3*index+2]])
        self._g_value_per_hdd.append(['15000', values[len(values)-1]])

      construction_url = f'{self._base_url_construction}{standard["constructions_location"]}'
      with urllib.request.urlopen(construction_url) as json_file:
        cases = json.load(json_file)['tables']['surface_thermal_transmittance']['table']
        # W/m2K
        for case in cases:
          surface = \
            ConstructionHelper().nrcan_surfaces_types_to_hub_types[f"{case['surface']}_{case['boundary_condition']}"]
          thermal_transmittance_per_hdd = []
          text = case['formula']
          values = ''.join([o for o in list(text) if o not in punc]).split()
          for index in range(int((len(values) - 1)/3)):
            thermal_transmittance_per_hdd.append([values[3*index+1], values[3*index+2]])
          thermal_transmittance_per_hdd.append(['15000', values[len(values)-1]])
          self._thermal_transmittance_per_hdd_and_surface[surface] = thermal_transmittance_per_hdd

  def _load_constructions(self, window_ratio_standard, construction_standard):
    constructions = []
    # todo: we need to save the total transmittance somehow, we don't do it yet in our archetypes
    # todo: it has to be selected the specific thermal_transmittance from
    #  self._thermal_transmittance_per_hdd_and_surface and window_ratios from self._window_ratios for each standard case
    for i, surface_type in enumerate(self._thermal_transmittance_per_hdd_and_surface):
      constructions.append(Construction(i, surface_type, None, None, self._window_ratios))
    return constructions

  def _load_archetypes(self):
    archetypes = []
    archetype_id = 0
    for window_ratio_standard in self._metadata['nrcan']['standards_per_function']['standard']:
      for construction_standard in self._metadata['nrcan']['standards_per_period']['standard']:
        archetype_id += 1
        function = window_ratio_standard['@function']
        climate_zone = 'Montreal'
        construction_period = construction_standard['@period_of_construction']
        constructions = self._load_constructions(window_ratio_standard, construction_standard)
        archetypes.append(Archetype(archetype_id,
                          None,
                          function,
                          climate_zone,
                          construction_period,
                          constructions,
                          None,
                          None,
                          None,
                          None,
                          None,
                          None))
    return archetypes

  def names(self, category=None):
    """
    Get the catalog elements names
    :parm: for usage catalog category filter does nothing as there is only one category (usages)
    """
    _names = {'usages': []}
    for usage in self._content.usages:
      _names['usages'].append(usage.name)
    return _names

  def entries(self, category=None):
    """
    Get the catalog elements
    :parm: for usage catalog category filter does nothing as there is only one category (usages)
    """
    return self._content

  def get_entry(self, name):
    """
    Get one catalog element by names
    :parm: entry name
    """
    for usage in self._content.usages:
      if usage.name.lower() == name.lower():
        return usage
    raise IndexError(f"{name} doesn't exists in the catalog")



"""
Geometry helper
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""

import numpy as np


class GeometryHelper:
  """
  Geometry helper
  """
  @staticmethod
  def to_points_matrix(points):
    """
    Transform a point vector into a point matrix
    :param points: [x, y, z, x, y, z ...]
    :return: [[x,y,z],[x,y,z]...]
    """
    rows = points.size // 3
    points = points.reshape(rows, 3)
    return points

  @staticmethod
  def gml_surface_to_libs(surface):
    """
    Transform citygml surface names into hub names
    """
    if surface == 'WallSurface':
      return 'Wall'
    if surface == 'GroundSurface':
      return 'Ground'
    return 'Roof'

  @staticmethod
  def points_from_string(coordinates) -> np.ndarray:
    points = np.fromstring(coordinates, dtype=float, sep=' ')
    points = GeometryHelper.to_points_matrix(points)
    return points

  @staticmethod
  def remove_last_point_from_string(points):
    array = points.split(' ')
    res = " "
    return res.join(array[0:len(array) - 3])
